//*******************************************************************************
// attention 
// Copyright (C) 2018 MACNICA,Inc. All Rights Reserved.
//   This software is licensed "AS IS". 
//   Please perform use of this software by a user's own responsibility and expense.
//   It cannot guarantee in the maker side about the damage which occurred by the ab-
//   ility not to use or use this software, and all damage that occurred secondarily.
//*******************************************************************************

// synthesis translate_off
`timescale 1ns / 1ps
// synthesis translate_on

`default_nettype none

module isp_avmm_slave 
#(
    parameter DATA_WIDTH = 32,  // Data bit width
    parameter ADRS_WIDTH = 3,   // Address width
    parameter DEF_COEF_Y = 32'h3A68F46C,    // Y Coefficient
    parameter BIAS_R     = 32'd0,   // 0
    parameter BIAS_G     = 32'd0,   // 0
    parameter BIAS_B     = 32'd0,   // 0
    parameter GAIN_R     = 32'd68,  // 1.07 * 2^6 = 68
    parameter GAIN_G     = 32'd57,  // 0.90 * 2^6 = 57
    parameter GAIN_B     = 32'd67   // 1.05 * 2^6 = 67 
)
(
    // clock & reset
    input   wire                        clock,      // Clock.
    input   wire                        reset_n,    // Reset.
    // Parameter set
    output  reg     [DATA_WIDTH-1:0]    coef_y,
    
    output  reg     [DATA_WIDTH-1:0]    bias_r,
    output  reg     [DATA_WIDTH-1:0]    bias_g,
    output  reg     [DATA_WIDTH-1:0]    bias_b,
    
    output  reg     [DATA_WIDTH-1:0]    gain_r,
    output  reg     [DATA_WIDTH-1:0]    gain_g,
    output  reg     [DATA_WIDTH-1:0]    gain_b,
    // Avalon-MM Slave
    input   wire    [ADRS_WIDTH-1:0]    address,
    input   wire                        write,
    input   wire    [DATA_WIDTH-1:0]    writedata,
    input   wire                        read,
    output  reg     [DATA_WIDTH-1:0]    readdata
);
    always @(posedge clock or negedge reset_n) begin
        if (!reset_n) begin
            coef_y   <= DEF_COEF_Y; // default (1 / (255.0 / 1.1))
            bias_r   <= BIAS_R;     // 0
            bias_g   <= BIAS_G;     // 0
            bias_b   <= BIAS_B;     // 0
            gain_r   <= GAIN_R;     // 1.07 * 2^6 = 68
            gain_g   <= GAIN_G;     // 0.90 * 2^6 = 57
            gain_b   <= GAIN_B;     // 1.05 * 2^6 = 67 
            readdata <= 0;
        end else begin
            if(write) begin
                case (address)
                    3'h0 : coef_y <= writedata;
                    3'h1 : bias_r <= writedata;
                    3'h2 : bias_g <= writedata;
                    3'h3 : bias_b <= writedata;
                    3'h4 : gain_r <= writedata;
                    3'h5 : gain_g <= writedata;
                    3'h6 : gain_b <= writedata;
                endcase
            end else
            if(read) begin
                case (address)
                    3'h0 : readdata <= coef_y;
                    3'h1 : readdata <= gain_r;
                    3'h2 : readdata <= gain_g;
                    3'h3 : readdata <= gain_b;
                    3'h4 : readdata <= gain_r;
                    3'h5 : readdata <= gain_g;
                    3'h6 : readdata <= gain_b;
                endcase
            end
        end
    end

endmodule
`default_nettype wire
