//*******************************************************************************
// attention 
// Copyright (C) 2018 MACNICA,Inc. All Rights Reserved.
//   This software is licensed "AS IS". 
//   Please perform use of this software by a user's own responsibility and expense.
//   It cannot guarantee in the maker side about the damage which occurred by the ab-
//   ility not to use or use this software, and all damage that occurred secondarily.
//*******************************************************************************

// synthesis translate_off
`timescale 1ns / 1ps
// synthesis translate_on

`default_nettype none

module isp_color_balance
#(
    parameter BITS_SIMBOL = 8,  // Data bit width
    parameter SIMBOL_PARA = 3   // Simbols in parallel
)
(
    input  wire                                 clock,
    input  wire                                 reset_n,

    input  wire [BITS_SIMBOL-1:0]               bias_r,
    input  wire [BITS_SIMBOL-1:0]               bias_g,
    input  wire [BITS_SIMBOL-1:0]               bias_b,

    input  wire [BITS_SIMBOL-1:0]               gain_r,
    input  wire [BITS_SIMBOL-1:0]               gain_g,
    input  wire [BITS_SIMBOL-1:0]               gain_b,

    input  wire [BITS_SIMBOL*SIMBOL_PARA-1:0]   in_data,
    output wire [BITS_SIMBOL*SIMBOL_PARA-1:0]   out_data
);
    localparam  CALC_BITS = BITS_SIMBOL + 1;
    localparam  OFFSET = 6;

    wire [BITS_SIMBOL-1:0]  in_r;
    wire [BITS_SIMBOL-1:0]  in_g;
    wire [BITS_SIMBOL-1:0]  in_b;

    reg  [BITS_SIMBOL+1:0]  ofs_r;
    reg  [BITS_SIMBOL+1:0]  ofs_g;
    reg  [BITS_SIMBOL+1:0]  ofs_b;

    wire [CALC_BITS*2-1:0]  res_r;
    wire [CALC_BITS*2-1:0]  res_g;
    wire [CALC_BITS*2-1:0]  res_b;

    always @(posedge clock or negedge reset_n) begin
        if (!reset_n) begin
            ofs_r <= {(BITS_SIMBOL+2){1'b0}};
            ofs_g <= {(BITS_SIMBOL+2){1'b0}};
            ofs_b <= {(BITS_SIMBOL+2){1'b0}};
        end else begin
            ofs_r <= {{(2){bias_r[BITS_SIMBOL-1]}}, bias_r} + {2'b0, in_data[BITS_SIMBOL*3-1 : BITS_SIMBOL*2]};
            ofs_g <= {{(2){bias_g[BITS_SIMBOL-1]}}, bias_g} + {2'b0, in_data[BITS_SIMBOL*2-1 : BITS_SIMBOL*1]};
            ofs_b <= {{(2){bias_b[BITS_SIMBOL-1]}}, bias_b} + {2'b0, in_data[BITS_SIMBOL*1-1 : BITS_SIMBOL*0]};
        end
    end

    assign  in_r = (ofs_r[BITS_SIMBOL+1])? {(BITS_SIMBOL){1'b0}} : (ofs_r[BITS_SIMBOL])? {(BITS_SIMBOL){1'b1}} : ofs_r;
    assign  in_g = (ofs_g[BITS_SIMBOL+1])? {(BITS_SIMBOL){1'b0}} : (ofs_g[BITS_SIMBOL])? {(BITS_SIMBOL){1'b1}} : ofs_g;
    assign  in_b = (ofs_b[BITS_SIMBOL+1])? {(BITS_SIMBOL){1'b0}} : (ofs_b[BITS_SIMBOL])? {(BITS_SIMBOL){1'b1}} : ofs_b;

    // res_r = in_r * gain_r 
    lpm_mult    u_gain_r (
                .clock  ( clock ),
                .aclr   (~reset_n ),
                .dataa  ( {1'b0, in_r} ),
                .datab  ( {1'b0, gain_r} ),
                .result ( res_r ),
                .clken  ( 1'b1 ),
                .sclr   ( 1'b0 ),
                .sum    ( 1'b0 ));
    defparam
        u_gain_r.lpm_pipeline = 3,
        u_gain_r.lpm_representation = "SIGNED",
        u_gain_r.lpm_type = "LPM_MULT",
        u_gain_r.lpm_widtha = CALC_BITS,
        u_gain_r.lpm_widthb = CALC_BITS,
        u_gain_r.lpm_widthp = CALC_BITS*2;

    // res_g = in_g * gain_g 
    lpm_mult    u_gain_g (
                .clock  ( clock ),
                .aclr   (~reset_n ),
                .dataa  ( {1'b0, in_g} ),
                .datab  ( {1'b0, gain_g} ),
                .result ( res_g ),
                .clken  ( 1'b1 ),
                .sclr   ( 1'b0 ),
                .sum    ( 1'b0 ));
    defparam
        u_gain_g.lpm_pipeline = 3,
        u_gain_g.lpm_representation = "SIGNED",
        u_gain_g.lpm_type = "LPM_MULT",
        u_gain_g.lpm_widtha = CALC_BITS,
        u_gain_g.lpm_widthb = CALC_BITS,
        u_gain_g.lpm_widthp = CALC_BITS*2;

    // res_b = in_b * gain_b 
    lpm_mult    u_gain_b (
                .clock  ( clock ),
                .aclr   (~reset_n ),
                .dataa  ( {1'b0, in_b} ),
                .datab  ( {1'b0, gain_b} ),
                .result ( res_b ),
                .clken  ( 1'b1 ),
                .sclr   ( 1'b0 ),
                .sum    ( 1'b0 ));
    defparam
        u_gain_b.lpm_pipeline = 3,
        u_gain_b.lpm_representation = "SIGNED",
        u_gain_b.lpm_type = "LPM_MULT",
        u_gain_b.lpm_widtha = CALC_BITS,
        u_gain_b.lpm_widthb = CALC_BITS,
        u_gain_b.lpm_widthp = CALC_BITS*2;

    wire [BITS_SIMBOL-1:0]    out_r;
    wire [BITS_SIMBOL-1:0]    out_g;
    wire [BITS_SIMBOL-1:0]    out_b;
    
    assign out_r = (res_r[CALC_BITS*2-1:OFFSET] > 9'd255)? 9'd255 : res_r[BITS_SIMBOL+OFFSET-1:OFFSET];
    assign out_g = (res_g[CALC_BITS*2-1:OFFSET] > 9'd255)? 9'd255 : res_g[BITS_SIMBOL+OFFSET-1:OFFSET];
    assign out_b = (res_b[CALC_BITS*2-1:OFFSET] > 9'd255)? 9'd255 : res_b[BITS_SIMBOL+OFFSET-1:OFFSET];

    assign out_data = {out_r, out_g, out_b};
    
endmodule

`default_nettype wire