//*******************************************************************************
// attention 
// Copyright (C) 2018 MACNICA,Inc. All Rights Reserved.
//   This software is licensed "AS IS". 
//   Please perform use of this software by a user's own responsibility and expense.
//   It cannot guarantee in the maker side about the damage which occurred by the ab-
//   ility not to use or use this software, and all damage that occurred secondarily.
//*******************************************************************************

// synthesis translate_off
`timescale 1ns / 1ps
// synthesis translate_on

`default_nettype none

module isp_csc_muladd
#(
    parameter CALC_BITS = 18    // Data bit width
)
(
    input  wire                     clock,
    input  wire                     reset_n,

    input  wire [CALC_BITS-1:0]     coef_0,
    input  wire [CALC_BITS-1:0]     coef_1,
    input  wire [CALC_BITS-1:0]     coef_2,

    input  wire [CALC_BITS-1:0]     in_0,
    input  wire [CALC_BITS-1:0]     in_1,
    input  wire [CALC_BITS-1:0]     in_2,
    
    output reg  [CALC_BITS*2-1:0]   out
);

    wire [CALC_BITS*2-1:0]          out_0;
    wire [CALC_BITS*2-1:0]          out_1;
    wire [CALC_BITS*2-1:0]          out_2;
    
    // out_0 = in_0 * coef_0
    lpm_mult    int_coef_0 (
                .clock  ( clock ),
                .aclr   (~reset_n ),
                .dataa  ( in_0 ),
                .datab  ( coef_0 ),
                .result ( out_0 ),
                .clken  ( 1'b1 ),
                .sclr   ( 1'b0 ),
                .sum    ( 1'b0 ));
    defparam
        int_coef_0.lpm_pipeline = 3,
        int_coef_0.lpm_representation = "SIGNED",
        int_coef_0.lpm_type = "LPM_MULT",
        int_coef_0.lpm_widtha = CALC_BITS,
        int_coef_0.lpm_widthb = CALC_BITS,
        int_coef_0.lpm_widthp = CALC_BITS*2;
        
    // out_1 = in_1 * coef_1
    lpm_mult    int_coef_1 (
                .clock  ( clock ),
                .aclr   (~reset_n ),
                .dataa  ( in_1 ),
                .datab  ( coef_1 ),
                .result ( out_1 ),
                .clken  ( 1'b1 ),
                .sclr   ( 1'b0 ),
                .sum    ( 1'b0 ));
    defparam
        int_coef_1.lpm_pipeline = 3,
        int_coef_1.lpm_representation = "SIGNED",
        int_coef_1.lpm_type = "LPM_MULT",
        int_coef_1.lpm_widtha = CALC_BITS,
        int_coef_1.lpm_widthb = CALC_BITS,
        int_coef_1.lpm_widthp = CALC_BITS*2;    
    
    // out_2 = in_2 * coef_2
    lpm_mult    int_coef_2 (
                .clock  ( clock ),
                .aclr   (!reset_n ),
                .dataa  ( in_2 ),
                .datab  ( coef_2 ),
                .result ( out_2 ),
                .clken  ( 1'b1 ),
                .sclr   ( 1'b0 ),
                .sum    ( 1'b0 ));
    defparam
        int_coef_2.lpm_pipeline = 3,
        int_coef_2.lpm_representation = "SIGNED",
        int_coef_2.lpm_type = "LPM_MULT",
        int_coef_2.lpm_widtha = CALC_BITS,
        int_coef_2.lpm_widthb = CALC_BITS,
        int_coef_2.lpm_widthp = CALC_BITS*2;    

    // out  =  in_0 * coef_0 + in_1 * ,coef_1 + in_2 * coef_2
    always @(posedge clock or negedge reset_n) begin
        if (!reset_n) begin
            out <= {(CALC_BITS*2){1'b0}};
        end else begin
            out <= out_0 + out_1 + out_2;
        end
    end
    
endmodule

`default_nettype wire