//*******************************************************************************
// attention 
// Copyright (C) 2018 MACNICA,Inc. All Rights Reserved.
//   This software is licensed "AS IS". 
//   Please perform use of this software by a user's own responsibility and expense.
//   It cannot guarantee in the maker side about the damage which occurred by the ab-
//   ility not to use or use this software, and all damage that occurred secondarily.
//*******************************************************************************

// synthesis translate_off
`timescale 1ns / 1ps
// synthesis translate_on

`default_nettype none

module isp_gamma_corrector
#(
    parameter BITS_SIMBOL = 8,  // Data bit width
    parameter DELAY       = 12, // Delay
    parameter MEM_FILE    = "gamma10.mif"   // Gamma table file name
)
(
    input  wire                     clock,
    input  wire                     reset_n,

    input  wire [BITS_SIMBOL-1:0]    in_data,
    output wire [BITS_SIMBOL-1:0]    out_data
);
    wire    [BITS_SIMBOL-1:0]    gam_data;

    altsyncram  altsyncram_component (
                .address_a (in_data),
                .clock0 (clock),
                .q_a (gam_data),
                .aclr0 (1'b0),
                .aclr1 (1'b0),
                .address_b (1'b1),
                .addressstall_a (1'b0),
                .addressstall_b (1'b0),
                .byteena_a (1'b1),
                .byteena_b (1'b1),
                .clock1 (1'b1),
                .clocken0 (1'b1),
                .clocken1 (1'b1),
                .clocken2 (1'b1),
                .clocken3 (1'b1),
                .data_a ({8{1'b1}}),
                .data_b (1'b1),
                .eccstatus (),
                .q_b (),
                .rden_a (1'b1),
                .rden_b (1'b1),
                .wren_a (1'b0),
                .wren_b (1'b0));
    defparam
        altsyncram_component.address_aclr_a = "NONE",
        altsyncram_component.clock_enable_input_a = "BYPASS",
        altsyncram_component.clock_enable_output_a = "BYPASS",
        altsyncram_component.init_file = MEM_FILE,
        altsyncram_component.intended_device_family = "Cyclone V",
        altsyncram_component.lpm_hint = "ENABLE_RUNTIME_MOD=NO",
        altsyncram_component.lpm_type = "altsyncram",
        altsyncram_component.numwords_a = 2 ** BITS_SIMBOL,
        altsyncram_component.operation_mode = "ROM",
        altsyncram_component.outdata_aclr_a = "NONE",
        altsyncram_component.outdata_reg_a = "CLOCK0",
        altsyncram_component.widthad_a = BITS_SIMBOL,
        altsyncram_component.width_a = BITS_SIMBOL,
        altsyncram_component.width_byteena_a = 1;

    isp_delay
    #(
        .DATA_WIDTH (BITS_SIMBOL), 
        .DELAY      (DELAY-2)
    )   u_ctl_delay
    (
        .clock    ( clock ),
        .reset_n  ( reset_n ),
        .in_data  ( gam_data ),
        .out_data ( out_data )
    );

endmodule

`default_nettype wire