//*******************************************************************************
// attention 
// Copyright (C) 2018 MACNICA,Inc. All Rights Reserved.
//   This software is licensed "AS IS". 
//   Please perform use of this software by a user's own responsibility and expense.
//   It cannot guarantee in the maker side about the damage which occurred by the ab-
//   ility not to use or use this software, and all damage that occurred secondarily.
//*******************************************************************************

// synthesis translate_off
`timescale 1ns / 1ps
// synthesis translate_on

`default_nettype none

module isp_input 
#(
    parameter BITS_SIMBOL   = 8,    // Bits per symbol
    parameter SIMBOL_PARA   = 3,    // Simbols in parallel
    parameter DELAY         = 12    // Delay latency
)
(
    input   wire                                    clock,
    input   wire                                    reset_n,

    // Avalon-ST Sink
    input   wire                                    in_valid,
    input   wire                                    in_sop,
    input   wire                                    in_eop,
    input   wire    [BITS_SIMBOL*SIMBOL_PARA-1:0]   in_data,
    // Avalon-ST Source
    output  reg                                     out_packet,
    output  reg                                     out_valid,
    output  reg                                     out_sop,
    output  reg                                     out_eop,
    output  reg     [BITS_SIMBOL*SIMBOL_PARA-1:0]   out_data
);
    localparam TA_IDLE = 2'd0;
    localparam TA_CTRL = 2'd1;
    localparam TA_DATA = 2'd2;
    localparam PAD_BITS = BITS_SIMBOL - 4;

    reg  [1:0]                  src_avst;
    reg  [3:0]                  src_pos;
    reg  [15:0]                 width;
    reg  [15:0]                 height;
    reg  [1:0]                  dst_avst;
    reg  [3:0]                  dst_pos;


    wire [BITS_SIMBOL-1:0]      in_data_y;
    wire [BITS_SIMBOL-1:0]      in_data_v;
    wire [BITS_SIMBOL-1:0]      in_data_u;
    wire [3:0]                  ctl_delay;
    wire                        ctl_valid;
    wire                        ctl_sop;
    wire                        ctl_eop;
    wire                        ctl_pack;

    assign  in_data_u = in_data[BITS_SIMBOL*1-1 : BITS_SIMBOL*0];
    assign  in_data_v = in_data[BITS_SIMBOL*2-1 : BITS_SIMBOL*1];
    assign  in_data_y = in_data[BITS_SIMBOL*3-1 : BITS_SIMBOL*2];
    
    always @(posedge clock or negedge reset_n) begin
        if (!reset_n) begin
            src_avst    <= TA_IDLE;
            src_pos     <= 4'd0;
            width       <= 16'd0;
            height      <= 16'd0;
        end else begin
            if(in_valid) begin
                case(src_avst)
                    TA_IDLE: begin
                        src_pos <= 4'd0;
                        if(in_sop) begin
                            if(in_data_u[3:0] == 4'hF) begin
                                src_avst <= TA_CTRL;
                            end else
                            if(in_data_u[3:0] == 4'h0) begin
                                src_avst <= TA_DATA;
                            end
                        end
                    end
                    TA_CTRL: begin
                        if(src_pos == 4'd0) begin
                            width[15:4] <= {in_data_u[3:0], in_data_v[3:0], in_data_y[3:0]};
                            src_pos <= 4'd1;
                        end else
                        if(src_pos == 4'd1) begin
                            width[3:0]   <= {in_data_u[3:0]};
                            height[15:8] <= {in_data_v[3:0], in_data_y[3:0]};
                            src_pos <= 4'd2;
                        end else
                        if(src_pos == 4'd2) begin
                            height[7:0] <= {in_data_u[3:0], in_data_v[3:0]};
                            src_pos     <= 4'd0;
                            src_avst    <= TA_IDLE;
                        end else
                        if(in_eop) begin
                            src_avst    <= TA_IDLE;
                        end
                    end
                    TA_DATA: begin
                        if(in_eop) begin
                            src_avst <= TA_IDLE;
                        end
                    end
                endcase
            end
        end
    end

    isp_delay
    #(
        .DATA_WIDTH (4), 
        .DELAY      (DELAY-1)
    )   u_ctl_delay
    (
        .clock    ( clock ),
        .reset_n  ( reset_n ),
        .in_data  ( {in_valid, in_sop, in_eop, in_data[0]}),
        .out_data ( ctl_delay )
    );

    assign ctl_valid  = ctl_delay[3];
    assign ctl_sop    = ctl_delay[2];
    assign ctl_eop    = ctl_delay[1];
    assign ctl_pack   = ctl_delay[0];

    always @(posedge clock or negedge reset_n) begin
        if (!reset_n) begin
            dst_avst <= TA_IDLE;
            dst_pos  <= 4'd0;
        end else begin
            out_valid  <= ctl_valid;
            out_sop    <= ctl_sop;
            out_eop    <= ctl_eop;
            out_packet <= (dst_avst == TA_DATA)? 1'b1 : 1'b0;
            if(ctl_valid) begin
                case(dst_avst)
                    TA_IDLE: begin
                        dst_pos <= 4'd0;
                        if(ctl_sop) begin
                            if(ctl_pack) begin
                                out_data <= {{(BITS_SIMBOL*SIMBOL_PARA-4){1'b0}}, 4'hF};
                                dst_avst <= TA_CTRL;
                            end else begin
                                out_data <= {(BITS_SIMBOL*SIMBOL_PARA){1'b0}};
                                dst_avst <= TA_DATA;
                            end
                        end
                    end
                    TA_CTRL: begin
                        if(dst_pos == 4'd0) begin
                            out_data <= {{(PAD_BITS){1'b0}}, width[7:4], {(PAD_BITS){1'b0}}, width[11:8], {(PAD_BITS){1'b0}}, width[15:12]};
                            dst_pos  <= 4'd1;
                        end else
                        if(dst_pos == 4'd1) begin
                            out_data <= {{(PAD_BITS){1'b0}}, height[11:8], {(PAD_BITS){1'b0}}, height[15:12], {(PAD_BITS){1'b0}}, width[3:0]};
                            dst_pos  <= 4'd2;
                        end else
                        if(dst_pos == 4'd2) begin
                            out_data <= {{(BITS_SIMBOL-2){1'b0}}, 2'h2, {(PAD_BITS){1'b0}}, height[3:0], {(PAD_BITS){1'b0}}, height[7:4]};
                            dst_pos  <= 4'd0;
                            dst_avst <= TA_IDLE;
                        end else
                        if(ctl_eop) begin
                            out_data <= 24'b0;
                            dst_avst <= TA_IDLE;
                        end
                    end
                    TA_DATA: begin
                        if(ctl_eop) begin
                            out_data <= 24'b0;
                            dst_avst <= TA_IDLE;
                        end
                    end
                endcase
            end
        end
    end
            
endmodule

`default_nettype wire