//*******************************************************************************
// attention 
// Copyright (C) 2018 MACNICA,Inc. All Rights Reserved.
//   This software is licensed "AS IS". 
//   Please perform use of this software by a user's own responsibility and expense.
//   It cannot guarantee in the maker side about the damage which occurred by the ab-
//   ility not to use or use this software, and all damage that occurred secondarily.
//*******************************************************************************

// synthesis translate_off
`timescale 1ns / 1ps
// synthesis translate_on

`default_nettype none

module isp_output 
#(
    parameter LATENCY = 12,                     // Latency cycle
    parameter SRC_WIDTH = 24,                   // Data bit width
    parameter FIFO_DEPTH = 5                    // FIFO depth bit width
)
(
    // clock & reset
    input  wire                     clock,      // Clock.
    input  wire                     reset_n,    // Reset.
    // avalon-st sink (Latency:n)
    output wire                     in_ready,   // Buffer status is full.
    input  wire                     in_valid,   // Write enable.
    input  wire [SRC_WIDTH-1: 0]    in_data,    // Write data.
    input  wire                     in_sop,     // Write data.
    input  wire                     in_eop,     // Write data.
    // avalon-st source (Latency:1)
    input  wire                     out_ready,  // Read enable.
    output reg                      out_valid,  // Buffer status is empty.
    output reg  [SRC_WIDTH-1: 0]    out_data,   // Read data.
    output reg                      out_sop,    // Read data.
    output reg                      out_eop     // Read data.
);

    // Local parameters.
    localparam FIFO_SIZE  = 2 ** FIFO_DEPTH;

    reg     [SRC_WIDTH + 1:0]   fifo_mem    [FIFO_SIZE - 1:0];

    // FIFO buffer memory counters and pointers.
    reg  [FIFO_DEPTH:0]     wr_count;
    reg  [FIFO_DEPTH:0]     rd_count;
    wire [FIFO_DEPTH - 1:0] wr_pos;
    wire [FIFO_DEPTH - 1:0] rd_pos;
    wire [FIFO_DEPTH:0]     usew;

    assign  wr_pos  = wr_count[FIFO_DEPTH - 1:0];
    assign  rd_pos  = rd_count[FIFO_DEPTH - 1:0];
    assign  usew    = wr_count - rd_count;

    always @(posedge clock or negedge reset_n) begin
        if (!reset_n) begin
            wr_count    <= 0;
            rd_count    <= 0;
            out_valid   <= 1'b0;
            out_sop     <= 1'b0;
            out_eop     <= 1'b0;
            out_data    <= 0;
        end else begin
            if (in_valid) begin
                fifo_mem[wr_pos]    <= ({{in_sop, in_eop}, in_data});
                wr_count            <= wr_count + {{(FIFO_DEPTH-1){1'b0}}, 1'b1};
            end
            if (out_ready && (usew != 0)) begin
                out_valid   <= 1'b1;
                out_sop     <= fifo_mem[rd_pos][SRC_WIDTH+1];
                out_eop     <= fifo_mem[rd_pos][SRC_WIDTH];
                out_data    <= fifo_mem[rd_pos][SRC_WIDTH-1 : 0];
                rd_count    <= rd_count + {{(FIFO_DEPTH-1){1'b0}}, 1'b1};
            end else begin
                out_valid   <= 1'b0;
            end
        end
    end

    assign  in_ready    = ((usew >= (FIFO_SIZE - LATENCY))) ? (1'b0) : (1'b1);

endmodule
`default_nettype wire
