//*******************************************************************************
// attention 
// Copyright (C) 2018 MACNICA,Inc. All Rights Reserved.
//   This software is licensed "AS IS". 
//   Please perform use of this software by a user's own responsibility and expense.
//   It cannot guarantee in the maker side about the damage which occurred by the ab-
//   ility not to use or use this software, and all damage that occurred secondarily.
//*******************************************************************************

// synthesis translate_off
`timescale 1ns / 1ps
// synthesis translate_on

`default_nettype none

// Y =  0.256788R + 0.504129G + 0.097906B +  16
// U = -0.148223R - 0.290993G + 0.439216B + 128
// V =  0.439216R - 0.367788G - 0.071427B + 128

module isp_rgb2yuv
#(
    parameter BITS_SIMBOL = 8,  // Data bit width
    parameter SIMBOL_PARA = 3   // Simbols in parallel
)
(
    input  wire                                 clock,
    input  wire                                 reset_n,

    input  wire [BITS_SIMBOL*SIMBOL_PARA-1:0]   in_data,
    output wire [BITS_SIMBOL*SIMBOL_PARA-1:0]   out_data
);
    localparam CALC_BITS = 18;
    localparam COEF_Y_R  = 18'h020DE;   //  0.256788 * 2^15 =  8414 = 20DE
    localparam COEF_Y_G  = 18'h04087;   //  0.504129 * 2^15 = 16519 = ‭4087‬
    localparam COEF_Y_B  = 18'h00C88;   //  0.097906 * 2^15 =  3208 = 0C88
    localparam COEF_U_R  = 18'hFED07;   // -0.148223 * 2^15 = -4857 = FFFFFFFFFFFFED07
    localparam COEF_U_G  = 18'hFDAC1;   // -0.290993 * 2^15 = -9535 = FFFFFFFFFFFFDAC1
    localparam COEF_U_B  = 18'h03838;   //  0.439216 * 2^15 = 14392 = 3838‬
    localparam COEF_V_R  = 18'h03838;   //  0.439216 * 2^15 =  14392 = ‭3838‬
    localparam COEF_V_G  = 18'hFD0EC;   // -0.367788 * 2^15 = -12052 = ‭FFFFFFFFFFFFD0EC‬
    localparam COEF_V_B  = 18'hFF6DB;   // -0.071427 * 2^15 =  -2341 = ‭FFFFFFFFFFFFF6DB‬

    wire [CALC_BITS-1:0]    in_r;
    wire [CALC_BITS-1:0]    in_g;
    wire [CALC_BITS-1:0]    in_b;

    assign  in_r = {{(CALC_BITS-BITS_SIMBOL){1'b0}}, in_data[BITS_SIMBOL*3-1 : BITS_SIMBOL*2]};
    assign  in_g = {{(CALC_BITS-BITS_SIMBOL){1'b0}}, in_data[BITS_SIMBOL*2-1 : BITS_SIMBOL*1]};
    assign  in_b = {{(CALC_BITS-BITS_SIMBOL){1'b0}}, in_data[BITS_SIMBOL*1-1 : BITS_SIMBOL*0]};

    wire [CALC_BITS*2-1:0]  res_y;
    wire [CALC_BITS*2-1:0]  res_u;
    wire [CALC_BITS*2-1:0]  res_v;
    
    // Y =  0.256788R + 0.504129G + 0.097906B
    isp_csc_muladd
    #(
        .CALC_BITS (CALC_BITS)
    )   u_csc_y
    (
        .clock      ( clock ),
        .reset_n    ( reset_n ),
        .coef_0     ( COEF_Y_R ),
        .coef_1     ( COEF_Y_G ),
        .coef_2     ( COEF_Y_B ),
        .in_0       ( in_r ),
        .in_1       ( in_g ),
        .in_2       ( in_b ),
        
        .out        ( res_y )
    );
    // U = -0.148223R - 0.290993G + 0.439216B
    isp_csc_muladd
    #(
        .CALC_BITS (CALC_BITS)
    )   u_csc_u
    (
        .clock      ( clock ),
        .reset_n    ( reset_n ),
        .coef_0     ( COEF_U_R ),
        .coef_1     ( COEF_U_G ),
        .coef_2     ( COEF_U_B ),
        .in_0       ( in_r ),
        .in_1       ( in_g ),
        .in_2       ( in_b ),
        
        .out        ( res_u )
    );

    // V =  0.439216R - 0.367788G - 0.071427B
    isp_csc_muladd
    #(
        .CALC_BITS (CALC_BITS)
    )   u_csc_v
    (
        .clock      ( clock ),
        .reset_n    ( reset_n ),
        .coef_0     ( COEF_V_R ),
        .coef_1     ( COEF_V_G ),
        .coef_2     ( COEF_V_B ),
        .in_0       ( in_r ),
        .in_1       ( in_g ),
        .in_2       ( in_b ),
        
        .out        ( res_v )
    );

    reg  signed [CALC_BITS*2-1:0]  reg_y;
    reg  signed [CALC_BITS*2-1:0]  reg_u;
    reg  signed [CALC_BITS*2-1:0]  reg_v;
    // 16.0 -> I[20:0].F[14:0]  16.0 X 2^15 = 80000
    // 128.0 -> I[20:0].F[14:0]  128.0 X 2^15 = 400000
    always @(posedge clock or negedge reset_n) begin
        if (!reset_n) begin
            reg_y <= {(CALC_BITS*2){1'b0}};
            reg_u <= {(CALC_BITS*2){1'b0}};
            reg_v <= {(CALC_BITS*2){1'b0}};
        end else begin
            reg_y <= res_y + 36'h80000;
            reg_u <= res_u + 36'h400000;
            reg_v <= res_v + 36'h400000;
        end
    end

    wire signed [21:0]  ofs_y;
    wire signed [21:0]  ofs_u;
    wire signed [21:0]  ofs_v;

    assign ofs_y = reg_y[CALC_BITS*2-1:15];
    assign ofs_u = reg_u[CALC_BITS*2-1:15];
    assign ofs_v = reg_v[CALC_BITS*2-1:15];

    wire signed [BITS_SIMBOL-1:0]  out_y;
    wire signed [BITS_SIMBOL-1:0]  out_u;
    wire signed [BITS_SIMBOL-1:0]  out_v;

    assign out_y = ofs_y[BITS_SIMBOL-1:0];
    assign out_u = ofs_u[BITS_SIMBOL-1:0];
    assign out_v = ofs_v[BITS_SIMBOL-1:0];
    
    assign out_data = {out_y, out_v, out_u};

    
endmodule

`default_nettype wire