//==============================================================================
//	File Name	: AM_SLAVE.v
//	Version		: 1.1
//	Date		: 2021/06/02
//	Model		: AM_SLAVE
//
//==============================================================================

module AM_SLAVE (
	input	wire				RST_N			,
	// -- Avalon_MM Slave -- //
	input	wire				ASCLK			,
	input	wire				IAS_WR			,
	input	wire	[31:0]		IAS_WDATA		,
	input	wire				IAS_RD			,
	output	wire	[31:0]		OAS_RDATA		,
	output	wire				OAS_RVLD		,
	input	wire	[1:0]		IAS_ADDR		,
	input	wire	[3:0]		IAS_BYEN		,
	output	wire				OAS_WAITREQ		,
	// -- Register -- //
	output						OREG_WEN		,
	input						IREG_WDNBNK_VLD	,
	input	[1:0]				IREG_WDNBNK		,
	output	[31:0]				OREG_BASE_ADDR
);

//==============================================================================
// Parameter
//==============================================================================

//==============================================================================
// Register & Wire
//==============================================================================
reg					as_wr		;
reg		[31:0]		as_wdata	;
reg		[1:0]		as_addr		;
reg		[3:0]		as_byen		;

wire	[31:0]		reg00_w		;
reg		[31:0]		reg00		;
wire	[31:0]		reg01		;
wire	[31:0]		reg02_w		;
reg		[31:0]		reg02		;

reg		[2:0]		wdnbnk_vld_pre;
wire				wdnbnk_vld_w;
reg					wdnbnk_vld	;
reg		[1:0]		wdnbnk		;

wire	[31:0]		rdata		;
reg		[31:0]		as_rdata	;
reg					as_rvld		;


//==============================================================================
// Description of Circuit
//==============================================================================

// Input FF
always @(posedge ASCLK or negedge RST_N) begin
	if (~RST_N) begin
			as_wr		<= 1'b0;
			as_wdata	<= 32'd0;
			as_addr		<= 2'd0;
			as_byen		<= 4'd0;
	end
	else begin
			as_wr		<= IAS_WR;
			as_wdata	<= IAS_WDATA;
			as_addr		<= IAS_ADDR;
			as_byen		<= IAS_BYEN;
	end
end


//----------------------------------------------------------
// WRITE
//----------------------------------------------------------


// addr = 0

generate
genvar i;
	for (i=0; i<=3; i=i+1)
	begin : g0
		assign reg00_w[i*8+7:i*8]	= (as_byen[i])? as_wdata[i*8+7:i*8] : reg00[i*8+7:i*8];
	end
endgenerate

always @(posedge ASCLK or negedge RST_N) begin
	if (~RST_N) 							reg00	<= 32'd0;
	else if (as_addr==2'b00 & as_wr)		reg00	<= reg00_w;
	else									reg00	<= reg00;
end

assign OREG_WEN	= reg00[0];

// addr = 1
// Read only


// addr = 2
generate
genvar j;
	for (j=0; j<=3; j=j+1)
	begin : g1
		assign reg02_w[j*8+7:j*8]	= (as_byen[j])? as_wdata[j*8+7:j*8] : reg02[j*8+7:j*8];
	end
endgenerate


always @(posedge ASCLK or negedge RST_N) begin
	if (~RST_N) 							reg02	<= 32'b00;
	else if (as_addr==2'b10 & as_wr)		reg02	<= reg02_w;
	else									reg02	<= reg02;
end

assign OREG_BASE_ADDR = reg02;


//----------------------------------------------------------
// READ
//----------------------------------------------------------
// addr = 1
// CDC
always @(posedge ASCLK or negedge RST_N) begin
	if (~RST_N)		wdnbnk_vld_pre	<= 3'b000;
	else			wdnbnk_vld_pre	<= {wdnbnk_vld_pre[1:0], IREG_WDNBNK_VLD};
end

assign wdnbnk_vld_w = (wdnbnk_vld_pre[2:1]==2'b01)? 1'b1 : 1'b0;	// rising edge

always @(posedge ASCLK or negedge RST_N) begin
	if (~RST_N) begin				wdnbnk_vld	<= 1'b0;
									wdnbnk		<= 2'b00;
	end
	else if (reg00[0]) begin
		if (wdnbnk_vld_w) begin			wdnbnk_vld	<= 1'b1;
										wdnbnk		<= IREG_WDNBNK;
		end
		else begin						wdnbnk_vld	<= wdnbnk_vld;
										wdnbnk		<= wdnbnk;
		end
	end
	else begin
										wdnbnk_vld	<= 1'b0;
										wdnbnk		<= wdnbnk;
	end
end

assign reg01 = {28'd0, wdnbnk_vld, wdnbnk};


// read select
assign rdata =	(IAS_ADDR==2'b00)? reg00 :
				(IAS_ADDR==2'b01)? reg01 :
				(IAS_ADDR==2'b10)? reg02 :
				32'd0;

always @(posedge ASCLK or negedge RST_N) begin
	if (~RST_N) begin			as_rdata	<= 32'd0;
								as_rvld		<= 1'b0;
	end
	else if (IAS_RD) begin		as_rdata	<= rdata;
								as_rvld		<= 1'b1;
	end
	else begin					as_rdata	<= 32'd0;
								as_rvld		<= 1'b0;
	end
end

assign OAS_RDATA = as_rdata;
assign OAS_RVLD = as_rvld;


assign OAS_WAITREQ	= 1'b0;

endmodule
