#!/usr/bin/env python3

import numpy			as np
import tkinter			as tk
import cam_init			as ci
import threading
import configparser
import os

PRD_POLLING				= 3000

ADR_ISP_CTRL00			= 0x5000
ADR_ISP_CTRL01			= 0x5001
ADR_ISP_CTRL02			= 0x5002
ADR_ISP_CTRL03			= 0x5003
ADR_ISP_CTRL05			= 0x5005
ADR_AEC_EXP_U			= 0x3500
ADR_AEC_EXP_M			= 0x3501
ADR_AEC_EXP_L			= 0x3502
ADR_MAN_CTRL			= 0x3503
ADR_MAN_CTRL			= 0x3503
ADR_AGC_GAIN_U			= 0x350a
ADR_AGC_GAIN_L			= 0x350b
ADR_AEC_GAIN_CEILING_U	= 0x3a18
ADR_AEC_GAIN_CEILING_L	= 0x3a19
ADR_AEC_VPT_H			= 0x3a11
ADR_AEC_VPT_L			= 0x3a1f
ADR_AEC_WPT				= 0x3a0f
ADR_AEC_BPT				= 0x3a10
ADR_AEC_WPT2			= 0x3a1b
ADR_AEC_BPT2			= 0x3a1e
ADR_LUM_AVG				= 0x5693

INI_LENC_EN				= 0x00
INI_BC_EN				= 0x01
INI_WC_EN				= 0x01
INI_AWB_EN				= 0x01
INI_WIN_EN				= 0x01
INI_OTP_EN				= 0x00
INI_AWB_GAIN_EN			= 0x01
INI_AGC_MAN				= 0x00
INI_AEC_MAN				= 0x00
INI_VTS_MAN				= 0x00
INI_GLAT_DLY			= 0x00
INI_AGC_GAIN			= 0x010
INI_AEC_EXP				= 0x04000
INI_AEC_GAIN_CEILING	= 0x080
INI_AEC_VPT_H			= 0x40
INI_AEC_VPT_L			= 0x08
INI_AEC_WPT				= 0x20
INI_AEC_BPT				= 0x18
INI_AEC_WPT2			= 0x20
INI_AEC_BPT2			= 0x18

AEC_AUTO				= 0
AEC_MANUAL				= 1
AGC_AUTO				= 0
AGC_MANUAL				= 1

INI_FILE				= "./cam_set.ini"
REG_FILE				= "./cam_set.reg"


### ISP general control ########################################

def set_isp0():
	val_lenc		= int( enb_lenc	.get()	)
	val_bc			= int( enb_bc	.get()	)
	val_wc			= int( enb_wc	.get()	)
	val_isp0		= (val_lenc << 7) + (val_bc << 2) + (val_wc << 1)
	ci.cis_i2c_write( ADR_ISP_CTRL00,	val_isp0 )

def set_isp1():
	val_awb			= int( enb_awb	.get()	)
	ci.cis_i2c_write( ADR_ISP_CTRL01,	val_awb	 )

def set_isp2():
	val_win			= INI_WIN_EN
	val_otp			= INI_OTP_EN
	val_awb_gain	= int( enb_awb_gain.get() )
	val_isp2		= (val_win << 6) + (val_otp << 1) + val_awb_gain
	ci.cis_i2c_write( ADR_ISP_CTRL02,	val_isp2 )

def def_isp():
	enb_lenc		.set( INI_LENC_EN		)
	enb_bc			.set( INI_BC_EN			)
	enb_wc			.set( INI_WC_EN			)
	enb_awb			.set( INI_AWB_EN		)
	enb_awb_gain	.set( INI_AWB_GAIN_EN	)
	chg_isp( 0 )

def chg_isp( n ):
	set_isp0()
	set_isp1()
	set_isp2()

def top_isp( self ):
	global			frm_isp
	global			enb_lenc
	global			enb_bc
	global			enb_wc
	global			enb_awb
	global			enb_awb_gain

	frm_isp			= tk.Frame( self.master,
						relief		= tk.SUNKEN,
						bd			= 2 )

	lbl_top			= tk.Label( frm_isp,
						font		= ("", 9, "bold"),
						text		= "ISP General controls" )
	enb_lenc		= tk.BooleanVar(
						value		= ini_lenc )
	chk_lenc		= tk.Checkbutton( frm_isp,
						text		= "Lens Correction (LENC)",
						variable	= enb_lenc,
						command		= set_isp0 )
	enb_bc			= tk.BooleanVar(
						value		= ini_bc )
	chk_bc			= tk.Checkbutton( frm_isp,
						text		= "Defect Pixel Cancellation (DPC.Black)",
						variable	= enb_bc,
						command		= set_isp0 )
	enb_wc			= tk.BooleanVar(
						value		= ini_wc )
	chk_wc			= tk.Checkbutton( frm_isp,
						text		= "Defect Pixel Cancellation (DPC.White)",
						variable	= enb_wc,
						command		= set_isp0 )
	enb_awb			= tk.BooleanVar(
						value		= ini_awb )
	chk_awb			= tk.Checkbutton( frm_isp,
						text		= "Auto White Balance (AWB)",
						variable	= enb_awb,
						command		= set_isp1 )
	enb_awb_gain	= tk.BooleanVar(
						value		= ini_awb_gain )
	chk_awb_gain	= tk.Checkbutton( frm_isp,
						text		= "Auto White Balance Gain (AWB.Gain)",
						variable	= enb_awb_gain,
						command		= set_isp2 )

	chg_isp( 0 )
	btn_def			= tk.Button	( frm_isp,
						text		= "Default",
						command		= def_isp )

	lbl_top			.pack( side	= tk.TOP,		fill	= tk.Y,
						padx	= 5,	pady	= (0,5)	)
	chk_lenc		.pack( side	= tk.TOP,		anchor	= tk.W,
						padx	= 5 )
	chk_bc			.pack( side	= tk.TOP,		anchor	= tk.W,
						padx	= 5 )
	chk_wc			.pack( side	= tk.TOP,		anchor	= tk.W,
						padx	= 5 )
	chk_awb			.pack( side	= tk.TOP,		anchor	= tk.W,
						padx	= 5 )
	chk_awb_gain	.pack( side	= tk.TOP,		anchor	= tk.W,
						padx	= 5,	pady	= (0,5) )
	btn_def			.pack( side	= tk.BOTTOM,
						padx	= 5,	pady	= (0,10) )


### Direct Register Access #####################################

def wr_reg():
	val_adr			= str( val_reg_adr	.get()	)
	val_dat			= str( val_reg_dat	.get()	)
	adr				= int( val_adr,		16		)
	dat				= int( val_dat,		16		)
	ci.cis_i2c_write( adr,	dat	)

def rd_reg():
	val_reg_dat		.set( "" )
	val_adr			= str( val_reg_adr	.get()	)
	adr				= int( val_adr,		16		)
	val_dat			= ci.cis_i2c_read ( adr	)
	dat				= ( '{:02X}'.format(val_dat) )
	val_reg_dat		.set( dat )

def	wr_file():
	if os.path.isfile( REG_FILE ):
		fp				= open( REG_FILE, 'r' )
		rdlines			= fp.readlines()
		for line in rdlines:
			if not "#" in line:
				reg			= line.split( ',' )
				ci.cis_i2c_write( int(reg[0],16), int(reg[1],16) )
		fp.close()

	else:
		print( "Error: Regster setting file(", REG_FILE, ") not found!" )

def top_reg( self ):
	global			frm_reg
	global			val_reg_adr
	global			val_reg_dat

	# Frame
	frm_reg			= tk.Frame( self.master,
						relief		= tk.SUNKEN,
						bd			= 2 )
	lbl_top			= tk.Label( frm_reg,
						font		= ("", 9, "bold"),
						text		= "Direct Register Access" )

	# sub-Frame: Address & Data field
	frm_sub			= tk.Frame( frm_reg )
	val_reg_adr		= tk.StringVar()
	val_reg_dat		= tk.StringVar()
	lbl_adr			= tk.Label( frm_sub,
						text		= "Address" )
	lbl_adr_dsc		= tk.Label( frm_sub,
						text		= " (16bit) : 0x" )
	lbl_dat			= tk.Label( frm_sub,
						text		= "Data" )
	lbl_dat_dsc		= tk.Label( frm_sub,
						text		= " (8bit) : 0x" )
	ent_adr			= tk.Entry( frm_sub,
						textvariable= val_reg_adr,
						width		= 4 )
	ent_dat			= tk.Entry( frm_sub,
						textvariable= val_reg_dat,
						width		= 4 )

	lbl_adr			.grid( row	= 0,	column	= 0,	sticky	= tk.W )
	lbl_adr_dsc		.grid( row	= 0,	column	= 1,	sticky	= tk.E )
	ent_adr			.grid( row	= 0,	column	= 2,	sticky	= tk.W )
	lbl_dat			.grid( row	= 1,	column	= 0,	sticky	= tk.W )
	lbl_dat_dsc		.grid( row	= 1,	column	= 1,	sticky	= tk.E )
	ent_dat			.grid( row	= 1,	column	= 2,	sticky	= tk.W )

	# sub-Frame: Write & Read button
	frm_btn			= tk.Frame( frm_reg )
	btn_wr			= tk.Button( frm_btn,
						text		= "Write",
						command		= wr_reg )
	btn_rd			= tk.Button( frm_btn,
						text		= "Read",
						command		= rd_reg )

	btn_wr			.pack( side	= tk.LEFT )
	btn_rd			.pack( side	= tk.LEFT )

	# sub-Frame: File Write
	frm_file		= tk.Frame( frm_reg )
	lbl_file		= tk.Label( frm_file,
						text		= "Write from File : cam_set.reg" )
	btn_file		= tk.Button( frm_file,
						text		= "Reg Write",
						command		= wr_file )

	lbl_file		.pack( side	= tk.TOP,
										pady    = (0,5) )
	btn_file		.pack( side	= tk.TOP )

	# Frame
	lbl_top			.pack( side	= tk.TOP,		fill	= tk.X,
						padx	= 5,	pady	= (0,5) )
	frm_sub			.pack( side	= tk.TOP,		anchor	= tk.CENTER,
						padx	= 5,	pady	= (0,5) )
	frm_btn			.pack( side	= tk.TOP,		anchor	= tk.CENTER,
						padx	= 5,	pady	= (0,5) )
	frm_file		.pack( side	= tk.TOP,		anchor	= tk.CENTER,
						padx	= 5,	pady	= (5,5) )


### AEC/AGC controls ###########################################

##  Manual control

def set_aec_agc():
	global			aec_man
	global			agc_man

	aec_man			= int( val_aec_man	.get() )
	agc_man			= int( val_agc_man	.get() )
	vts_man			= INI_VTS_MAN
	gain_lat_dly	= INI_GLAT_DLY
	val_man_ctrl	= (gain_lat_dly << 4) + (vts_man << 2) + (agc_man << 1) + aec_man

	ci.cis_i2c_write( ADR_MAN_CTRL,		val_man_ctrl )

	if( aec_man == AEC_MANUAL ):
		lbl_aec_exp		.config( state	= tk.NORMAL )
		scl_aec_exp		.config( state	= tk.NORMAL )
		lbl_aec_vpt		.config( state	= tk.DISABLED )
		lbl_aec_pt_1	.config( state	= tk.DISABLED )
		lbl_aec_pt_2	.config( state	= tk.DISABLED )
		lbl_aec_gc		.config( state	= tk.DISABLED )
		scl_aec_vpt_h	.config( state	= tk.DISABLED )
		scl_aec_wpt		.config( state	= tk.DISABLED )
		scl_aec_wpt2	.config( state	= tk.DISABLED )
		scl_aec_vpt_l	.config( state	= tk.DISABLED )
		scl_aec_bpt		.config( state	= tk.DISABLED )
		scl_aec_bpt2	.config( state	= tk.DISABLED )
		scl_aec_gc		.config( state	= tk.DISABLED )
	else:
		lbl_aec_exp		.config( state	= tk.DISABLED )
		scl_aec_exp		.config( state	= tk.DISABLED )
		lbl_aec_vpt		.config( state	= tk.NORMAL )
		lbl_aec_pt_1	.config( state	= tk.NORMAL )
		lbl_aec_pt_2	.config( state	= tk.NORMAL )
		lbl_aec_gc		.config( state	= tk.NORMAL )
		scl_aec_vpt_h	.config( state	= tk.NORMAL )
		scl_aec_wpt		.config( state	= tk.NORMAL )
		scl_aec_wpt2	.config( state	= tk.NORMAL )
		scl_aec_vpt_l	.config( state	= tk.NORMAL )
		scl_aec_bpt		.config( state	= tk.NORMAL )
		scl_aec_bpt2	.config( state	= tk.NORMAL )
		scl_aec_gc		.config( state	= tk.NORMAL )

	if( agc_man == AGC_MANUAL ):
		lbl_agc_gain	.config( state	= tk.NORMAL )
		scl_agc_gain	.config( state	= tk.NORMAL )
	else:
		lbl_agc_gain	.config( state	= tk.DISABLED )
		scl_agc_gain	.config( state	= tk.DISABLED )

def def_aec_agc():
	val_aec_man		.set( INI_AEC_MAN	)
	val_agc_man		.set( INI_AGC_MAN	)
	chg_aec_agc( 0 )

def chg_aec_agc( n ):
	set_aec_agc()


##  Manual setting

def set_aec_exp():
	aec_exp			= int( val_aec_exp	.get() )
	aec_exp_h		= (aec_exp >> 16)	& 0x0f
	aec_exp_m		= (aec_exp >>  8)	& 0xff
	aec_exp_l		=  aec_exp			& 0xff

	ci.cis_i2c_write( ADR_AEC_EXP_U,	aec_exp_h )
	ci.cis_i2c_write( ADR_AEC_EXP_M,	aec_exp_m )
	ci.cis_i2c_write( ADR_AEC_EXP_L,	aec_exp_l )

def def_aec_exp():
	val_aec_exp		.set( INI_AEC_EXP	)
	chg_aec_exp( 0 )

def chg_aec_exp( n ):
	set_aec_exp()

def get_aec_exp( self ):
	aec_exp_u		= ci.cis_i2c_read ( ADR_AEC_EXP_U )
	aec_exp_m		= ci.cis_i2c_read ( ADR_AEC_EXP_M )
	aec_exp_l		= ci.cis_i2c_read ( ADR_AEC_EXP_L )
	aec_exp			= (aec_exp_u << 16) + (aec_exp_m << 8) + aec_exp_l
	val_aec_exp		.set( aec_exp )

def set_agc_gain():
	agc_gain		= int( val_agc_gain	.get() )
	agc_gain_u		= agc_gain >> 8
	agc_gain_l		= agc_gain & 0xff

	ci.cis_i2c_write( ADR_AGC_GAIN_U,	agc_gain_u )
	ci.cis_i2c_write( ADR_AGC_GAIN_L,	agc_gain_l )

def def_agc_gain():
	val_agc_gain	.set( INI_AGC_GAIN )
	chg_agc_gain( 0 )

def chg_agc_gain( n ):
	set_agc_gain()

def get_agc_gain( self ):
	agc_gain_u		= ci.cis_i2c_read ( ADR_AGC_GAIN_U )
	agc_gain_l		= ci.cis_i2c_read ( ADR_AGC_GAIN_L )
	agc_gain		= (agc_gain_u << 8) + agc_gain_l
	val_agc_gain	.set( agc_gain )


##  AEC control

def set_aec_ctrl():
	aec_vpt_h		= int( val_aec_vpt_h.get() )
	aec_vpt_l		= int( val_aec_vpt_l.get() )
	aec_wpt			= int( val_aec_wpt	.get() )
	aec_bpt			= int( val_aec_bpt	.get() )
	aec_wpt2		= int( val_aec_wpt2	.get() )
	aec_bpt2		= int( val_aec_bpt2	.get() )

	ci.cis_i2c_write( ADR_AEC_VPT_H,	aec_vpt_h	)
	ci.cis_i2c_write( ADR_AEC_VPT_L,	aec_vpt_l	)
	ci.cis_i2c_write( ADR_AEC_WPT,		aec_wpt		)
	ci.cis_i2c_write( ADR_AEC_BPT,		aec_bpt		)
	ci.cis_i2c_write( ADR_AEC_WPT2,		aec_wpt2	)
	ci.cis_i2c_write( ADR_AEC_BPT2,		aec_bpt2	)

def def_aec_ctrl():
	val_aec_vpt_h	.set( INI_AEC_VPT_H	)
	val_aec_vpt_l	.set( INI_AEC_VPT_L	)
	val_aec_wpt		.set( INI_AEC_WPT	)
	val_aec_bpt		.set( INI_AEC_BPT	)
	val_aec_wpt2	.set( INI_AEC_WPT2	)
	val_aec_bpt2	.set( INI_AEC_BPT2	)
	chg_aec_ctrl( 0 )

def chg_aec_ctrl( n ):
	set_aec_ctrl()


##  AEC gain ceiling

def set_aec_gc():
	aec_gc			= int( val_aec_gc	.get() )
	aec_gc_h		= aec_gc >> 8
	aec_gc_l		= aec_gc & 0xff

	ci.cis_i2c_write( ADR_AEC_GAIN_CEILING_U,	aec_gc_h )
	ci.cis_i2c_write( ADR_AEC_GAIN_CEILING_L,	aec_gc_l )

def def_aec_gc():
	val_aec_gc		.set( INI_AEC_GAIN_CEILING )
	chg_aec_gc( 0 )

def chg_aec_gc( n ):
	set_aec_gc()


##  Luminance Average

def get_aec_avg( self ):
	aec_avg			= ci.cis_i2c_read ( ADR_LUM_AVG )
	val_aec_avg		.set( str(aec_avg) )


##  AEC/AGC controls 

def def_aec_top():
	def_aec_agc()
	def_aec_exp()
	def_agc_gain()
	def_aec_ctrl()
	def_aec_gc()

def top_aec( self ):
	global			frm_aec_top
	global			val_aec_man
	global			val_agc_man
	global			val_aec_vpt_h
	global			val_aec_wpt
	global			val_aec_wpt2
	global			val_aec_vpt_l
	global			val_aec_bpt
	global			val_aec_bpt2

	# Frame: AEC/AGC controls 
	frm_aec_top		= tk.Frame( self.master,
						relief		= tk.SUNKEN,
						bd			= 2 )
	lbl_top			= tk.Label( frm_aec_top,
						font		= ("", 9, "bold"),
						text		= "AEC/AGC controls" )

	btn_def			= tk.Button( frm_aec_top,
						text		= "Default",
						command		= def_aec_top )

	# sub.1-Frame: Manual control
	lfm_aec_man		= tk.LabelFrame( frm_aec_top,
						relief		= tk.GROOVE,
						text		= "AEC manual" )
	lfm_agc_man		= tk.LabelFrame( frm_aec_top,
						relief		= tk.GROOVE,
						text		= "AGC manual" )

	val_aec_man		= tk.IntVar(
						value		= ini_aec_man )
	val_agc_man		= tk.IntVar(
						value		= ini_agc_man )

	rbn_aec_auto	= tk.Radiobutton( lfm_aec_man,
						value		= 0,
						text		= "Auto",
						variable	= val_aec_man,
						command		= set_aec_agc )
	rbn_aec_man		= tk.Radiobutton( lfm_aec_man,
						value		= 1,
						text		= "Manual",
						variable	= val_aec_man,
						command		= set_aec_agc )
	rbn_agc_auto	= tk.Radiobutton( lfm_agc_man,
						value		= 0,
						text		= "Auto",
						variable	= val_agc_man,
						command		= set_aec_agc )
	rbn_agc_man		= tk.Radiobutton( lfm_agc_man,
						value		= 1,
						text		= "Manual",
						variable	= val_agc_man,
						command		= set_aec_agc )

	rbn_aec_auto	.pack( side	= tk.LEFT )
	rbn_aec_man		.pack( side	= tk.LEFT )
	rbn_agc_auto	.pack( side	= tk.LEFT )
	rbn_agc_man		.pack( side	= tk.LEFT )

	# sub.1-Frame: AEC control
	lfm_aec_ctrl	= tk.LabelFrame( frm_aec_top,
						relief		= tk.GROOVE,
						text		= "AEC control" )

	# sub.2-Frame: AEC control: High Limit
	global			scl_aec_vpt_h
	global			scl_aec_wpt
	global			scl_aec_wpt2

	frm_aec_ctrl_h	= tk.Frame( lfm_aec_ctrl,
						relief		= tk.RAISED,
						bd			= 2 )
	lbl_aec_ctrl_h	= tk.Label( frm_aec_ctrl_h,
						font		= ("",8),
						text		= "high\nside" )

	val_aec_vpt_h	= tk.DoubleVar(
						value		= ini_aec_vpt_h )
	val_aec_wpt		= tk.DoubleVar(
						value		= ini_aec_wpt )
	val_aec_wpt2	= tk.DoubleVar(
						value		= ini_aec_wpt2 )

	scl_aec_vpt_h	= tk.Scale( frm_aec_ctrl_h,
						fg			= "#FF0000",
						orient		= 'v',
						sliderlength= 10,
						from_		= 0xff,
						to			= 0x00,
						digits		= 3,
						resolution	= 1,
						variable	= val_aec_vpt_h,
						command		= chg_aec_ctrl )
	scl_aec_wpt		= tk.Scale( frm_aec_ctrl_h,
						fg			= "#FF0000",
						orient		= 'v',
						sliderlength= 10,
						from_		= 0xff,
						to			= 0x00,
						digits		= 3,
						resolution	= 1,
						variable	= val_aec_wpt,
						command		= chg_aec_ctrl )
	scl_aec_wpt2	= tk.Scale( frm_aec_ctrl_h,
						fg			= "#FF0000",
						orient		= 'v',
						sliderlength= 10,
						from_		= 0xff,
						to			= 0x00,
						digits		= 3,
						resolution	= 1,
						variable	= val_aec_wpt2,
						command		= chg_aec_ctrl )

	lbl_aec_ctrl_h	.pack( side	= tk.LEFT,	anchor	= tk.NW	)
	scl_aec_vpt_h	.pack( side	= tk.LEFT )
	scl_aec_wpt		.pack( side	= tk.LEFT )
	scl_aec_wpt2	.pack( side	= tk.LEFT )

	# sub.2-Frame: AEC control: Low Limit
	global			scl_aec_vpt_l
	global			scl_aec_bpt
	global			scl_aec_bpt2

	frm_aec_ctrl_l	= tk.Frame( lfm_aec_ctrl,
						relief		= tk.RAISED,
						bd			= 2 )
	lbl_aec_ctrl_l	= tk.Label( frm_aec_ctrl_l,
						font		= ("",8),
						text		= "low\nside" )

	val_aec_vpt_l	= tk.DoubleVar(
						value		= ini_aec_vpt_l )
	val_aec_bpt		= tk.DoubleVar(
						value		= ini_aec_bpt )
	val_aec_bpt2	= tk.DoubleVar(
						value		= ini_aec_bpt2 )

	scl_aec_vpt_l	= tk.Scale( frm_aec_ctrl_l,
						fg			= "#FF0000",
						orient		= 'v',
						sliderlength= 10,
						from_		= 0xff,
						to			= 0x00,
						digits		= 3,
						resolution	= 1,
						variable	= val_aec_vpt_l,
						command		= chg_aec_ctrl )
	scl_aec_bpt		= tk.Scale( frm_aec_ctrl_l,
						fg			= "#FF0000",
						orient		= 'v',
						sliderlength= 10,
						from_		= 0xff,
						to			= 0x00,
						digits		= 3,
						resolution	= 1,
						variable	= val_aec_bpt,
						command		= chg_aec_ctrl )
	scl_aec_bpt2	= tk.Scale( frm_aec_ctrl_l,
						fg			= "#FF0000",
						orient		= 'v',
						sliderlength= 10,
						from_		= 0xff,
						to			= 0x00,
						digits		= 3,
						resolution	= 1,
						variable	= val_aec_bpt2,
						command		= chg_aec_ctrl )

	lbl_aec_ctrl_l	.pack( side	= tk.LEFT,	anchor	= tk.SW	)
	scl_aec_vpt_l	.pack( side	= tk.LEFT )
	scl_aec_bpt		.pack( side	= tk.LEFT )
	scl_aec_bpt2	.pack( side	= tk.LEFT )

	# sub.1-Frame: AEC control
	global			val_aec_gc
	global			lbl_aec_vpt	
	global			lbl_aec_pt_1
	global			lbl_aec_pt_2
	global			lbl_aec_gc	
	global			scl_aec_gc

	val_aec_gc		= tk.DoubleVar(
						value		= ini_aec_gc )

	lbl_aec_lmt		= tk.Label( lfm_aec_ctrl,
						width		= 7,
						font		= ("",8),
						text		= "limit" )
	lbl_aec_vpt		= tk.Label( lfm_aec_ctrl,
						width		= 7,
						font		= ("",8),
						text		= "control\nzone" )
	lbl_aec_pt_1	= tk.Label( lfm_aec_ctrl,
						width		= 7,
						font		= ("",8),
						text		= "to\nstable" )
	lbl_aec_pt_2	= tk.Label( lfm_aec_ctrl,
						width		= 7,
						font		= ("",8),
						text		= "to\nunstable" )
	lbl_aec_gc		= tk.Label( lfm_aec_ctrl,
						font		= ("",8),
						text		= "gain\nceiling" )
	scl_aec_gc		= tk.Scale( lfm_aec_ctrl,
						fg			= "#FF0000",
						orient		= 'v',
						sliderlength= 10,
						from_		= 0x3ff,
						to			= 0x000,
						digits		= 4,
						resolution	= 1,
						variable	= val_aec_gc,
						command		= chg_aec_gc )

	lbl_aec_lmt		.grid( row	= 0,	column	= 0	)
	lbl_aec_vpt		.grid( row	= 0,	column	= 1	)
	lbl_aec_pt_1	.grid( row	= 0,	column	= 2	)
	lbl_aec_pt_2	.grid( row	= 0,	column	= 3	)
	lbl_aec_gc		.grid( row	= 0,	column	= 4,
						sticky	= tk.E )
	frm_aec_ctrl_h	.grid( row	= 1,	column	= 0,	columnspan	= 4	)
	frm_aec_ctrl_l	.grid( row	= 2,	column	= 0,	columnspan	= 4,
						pady	= (0,5)	)
	scl_aec_gc		.grid( row	= 1,	column	= 4,
						padx	= (0,5) )

	# sub.1-Frame: Manual setting
	global			val_aec_exp
	global			val_agc_gain
	global			lbl_aec_exp
	global			lbl_agc_gain
	global			scl_aec_exp
	global			scl_agc_gain

	lfm_exp_gain	= tk.LabelFrame( frm_aec_top,
						relief		= tk.GROOVE,
						text		= "Manual setting" )

	val_aec_exp		= tk.DoubleVar(
						value		= ini_aec_exp )
	val_agc_gain	= tk.DoubleVar(
						value		= ini_agc_gain )

	lbl_aec_exp		= tk.Label( lfm_exp_gain,
						font		= ("",8),
						text		= "Exposure" )
	lbl_agc_gain	= tk.Label( lfm_exp_gain,
						font		= ("",8),
						text		= "Gain" )
	scl_aec_exp		= tk.Scale( lfm_exp_gain,
						fg			= "#FF0000",
						orient		= 'v',
						sliderlength= 10,
						from_		= 0xfffff,
						to			= 0x00000,
						digits		= 6,
						resolution	= 1,
						variable	= val_aec_exp,
						command		= chg_aec_exp )
	scl_agc_gain	= tk.Scale( lfm_exp_gain,
						fg			= "#FF0000",
						orient		= 'v',
						sliderlength= 10,
						from_		= 0x3ff,
						to			= 0x000,
						digits		= 4,
						resolution	= 1,
						variable	= val_agc_gain,
						command		= chg_agc_gain )

	lbl_aec_exp		.grid( row	= 0,	column	= 0,
						sticky	= tk.E )
	lbl_agc_gain	.grid( row	= 0,	column	= 1,
						padx	= (0,5),
						sticky	= tk.E )
	scl_aec_exp		.grid( row	= 1,	column	= 0,
											pady	= (0,5) )
	scl_agc_gain	.grid( row	= 1,	column	= 1,
						padx	= (0,5),	pady	= (0,5) )
 
	# sub.1-Frame: Luminance Average
	global			val_aec_avg

	lfm_aec_avg		= tk.LabelFrame( frm_aec_top,
						relief		= tk.GROOVE,
						text		= "Luminance Average" )
	lbl_aec_val		= tk.Label( lfm_aec_avg,
						width		= 8,
						text		= "value" )
	val_aec_avg		= tk.StringVar()
	lbl_aec_avg		= tk.Label( lfm_aec_avg,
						width		= 4,
						fg			= "red",
						textvariable= val_aec_avg )

	lbl_aec_val		.pack( side	= tk.LEFT,
						padx	= 5,	pady	= 5	)
	lbl_aec_avg		.pack( side	= tk.RIGHT,
						padx	= 5,	pady	= 5	)

	chg_aec_agc		( 0 )
	chg_aec_ctrl	( 0 )
	chg_aec_gc		( 0 )
	chg_agc_gain	( 0 )
	chg_aec_exp		( 0 )

	# Frame: AEC/AGC controls
	lbl_top			.pack( side	= tk.TOP,		fill	= tk.X,
						padx	= 5,		pady	= (0,5) )
	btn_def			.pack( side	= tk.BOTTOM,
						padx	= 5,		pady	= (0,5)	)
	lfm_aec_ctrl	.pack( side	= tk.RIGHT,		fill	= tk.Y,
						padx	= (5,10),	pady	= (0,5) )
	lfm_aec_man		.pack( side	= tk.TOP,		fill	= tk.X,
						padx	= (10,5),	pady	= (0,5) )
	lfm_agc_man		.pack( side	= tk.TOP,		fill	= tk.X,
						padx	= (10,5),	pady	= (0,5)	)
	lfm_exp_gain	.pack( side	= tk.TOP,		fill	= tk.X,
						padx	= (10,5),	pady	= (0,5)	)
	lfm_aec_avg		.pack( side	= tk.TOP,		fill	= tk.BOTH,
						padx	= (10,5),	pady	= (0,5)	)
 

### Init File control ##########################################

def get_ini():
	global			ini_lenc
	global			ini_bc
	global			ini_wc
	global			ini_awb
	global			ini_awb_gain
	global			ini_aec_man
	global			ini_agc_man
	global			ini_agc_gain
	global			ini_aec_exp
	global			ini_aec_vpt_h
	global			ini_aec_vpt_l
	global			ini_aec_wpt
	global			ini_aec_wpt2
	global			ini_aec_bpt
	global			ini_aec_bpt2
	global			ini_aec_gc

	cfg_ini			= configparser.ConfigParser()

	if os.path.exists( INI_FILE	):
		cfg_ini			.read( INI_FILE	)

		ini_lenc		= int( cfg_ini["init"]["lenc_en"	] )
		ini_bc			= int( cfg_ini["init"]["bc_en"		] )
		ini_wc			= int( cfg_ini["init"]["wc_en"		] )
		ini_awb			= int( cfg_ini["init"]["awb_en"		] )
		ini_awb_gain	= int( cfg_ini["init"]["awb_gain_en"] )
		ini_aec_man		= int( cfg_ini["init"]["aec_man"	] )
		ini_agc_man		= int( cfg_ini["init"]["agc_man"	] )
		ini_agc_gain	= int( cfg_ini["init"]["agc_gain"	] )
		ini_aec_exp		= int( cfg_ini["init"]["aec_exp"	] )
		ini_aec_vpt_h	= int( cfg_ini["init"]["aec_vpt_h"	] )
		ini_aec_vpt_l	= int( cfg_ini["init"]["aec_vpt_l"	] )
		ini_aec_wpt		= int( cfg_ini["init"]["aec_wpt"	] )
		ini_aec_bpt		= int( cfg_ini["init"]["aec_bpt"	] )
		ini_aec_wpt2	= int( cfg_ini["init"]["aec_wpt2"	] )
		ini_aec_bpt2	= int( cfg_ini["init"]["aec_bpt2"	] )
		ini_aec_gc		= int( cfg_ini["init"]["aec_gain_cl"] )

	else:
		ini_lenc		= int( INI_LENC_EN			)
		ini_bc			= int( INI_BC_EN			)
		ini_wc			= int( INI_WC_EN			)
		ini_awb			= int( INI_AWB_EN			)
		ini_awb_gain	= int( INI_AWB_GAIN_EN		)
		ini_aec_man		= int( INI_AEC_MAN			)
		ini_agc_man		= int( INI_AGC_MAN			)
		ini_agc_gain	= int( INI_AGC_GAIN			)
		ini_aec_exp		= int( INI_AEC_EXP			)
		ini_aec_vpt_h	= int( INI_AEC_VPT_H	   	)
		ini_aec_vpt_l	= int( INI_AEC_VPT_L	    )
		ini_aec_wpt		= int( INI_AEC_WPT		    )
		ini_aec_bpt		= int( INI_AEC_BPT		    )
		ini_aec_wpt2	= int( INI_AEC_WPT2		    )
		ini_aec_bpt2	= int( INI_AEC_BPT2		    )
		ini_aec_gc		= int( INI_AEC_GAIN_CEILING )
		cfg_ini["init"]	= {}
		cfg_ini["init"]["lenc_en"		] = str( ini_lenc		)
		cfg_ini["init"]["bc_en"			] = str( ini_bc			)
		cfg_ini["init"]["wc_en"			] = str( ini_wc			)
		cfg_ini["init"]["awb_en"		] = str( ini_awb		)
		cfg_ini["init"]["awb_gain_en"	] = str( ini_awb_gain	)
		cfg_ini["init"]["aec_man"		] = str( ini_aec_man	)
		cfg_ini["init"]["agc_man"		] = str( ini_agc_man	)
		cfg_ini["init"]["agc_gain"		] = str( ini_agc_gain	)
		cfg_ini["init"]["aec_exp"		] = str( ini_aec_exp	)
		cfg_ini["init"]["aec_vpt_h"		] = str( ini_aec_vpt_h	)
		cfg_ini["init"]["aec_vpt_l"		] = str( ini_aec_vpt_l	)
		cfg_ini["init"]["aec_wpt"		] = str( ini_aec_wpt	)
		cfg_ini["init"]["aec_bpt"		] = str( ini_aec_bpt	)
		cfg_ini["init"]["aec_wpt2"		] = str( ini_aec_wpt2	)
		cfg_ini["init"]["aec_bpt2"		] = str( ini_aec_bpt2	)
		cfg_ini["init"]["aec_gain_cl"	] = str( ini_aec_gc		)
                                                     
		fp				= open( INI_FILE,	'w'	)
		cfg_ini			.write( fp	)

def put_ini():
	cfg_ini			= configparser.ConfigParser()
	cfg_ini			.read( INI_FILE	)

	cfg_ini["init"]["lenc_en"		] = str( int(enb_lenc		.get()) )
	cfg_ini["init"]["bc_en"			] = str( int(enb_bc			.get()) )
	cfg_ini["init"]["wc_en"			] = str( int(enb_wc			.get()) )
	cfg_ini["init"]["awb_en"		] = str( int(enb_awb		.get()) )
	cfg_ini["init"]["awb_gain_en"	] = str( int(enb_awb_gain	.get()) )
	cfg_ini["init"]["aec_man"		] = str( int(val_aec_man	.get()) )
	cfg_ini["init"]["agc_man"		] = str( int(val_agc_man	.get()) )
	cfg_ini["init"]["agc_gain"		] = str( int(val_agc_gain	.get()) )
	cfg_ini["init"]["aec_exp"		] = str( int(val_aec_exp	.get()) )
	cfg_ini["init"]["aec_vpt_h"		] = str( int(val_aec_vpt_h	.get()) )
	cfg_ini["init"]["aec_vpt_l"		] = str( int(val_aec_vpt_l	.get()) )
	cfg_ini["init"]["aec_wpt"		] = str( int(val_aec_wpt	.get()) )
	cfg_ini["init"]["aec_bpt"		] = str( int(val_aec_bpt	.get()) )
	cfg_ini["init"]["aec_wpt2"		] = str( int(val_aec_wpt2	.get()) )
	cfg_ini["init"]["aec_bpt2"		] = str( int(val_aec_bpt2	.get()) )
	cfg_ini["init"]["aec_gain_cl"	] = str( int(val_aec_gc		.get()) )

	fp				= open( INI_FILE,	'w'	)
	cfg_ini			.write( fp	)


### Application Top ############################################

class Application( tk.Frame ):

	def	__init__( self, master	= None ):
		super().__init__( master )

		get_ini()

		self.master.title( "Camera CIS: OV5647 settings" ) 
	
		top_isp		( self )
		top_aec		( self )
		top_reg		( self )
	
		frm_aec_top	.pack(
						side	= tk.RIGHT,
						anchor	= tk.NE,
						fill	= tk.Y )
		frm_isp		.pack(
						side	= tk.TOP,
						anchor	= tk.NW )
		frm_reg		.pack(
						side	= tk.TOP,
						anchor	= tk.NW,
						expand	= True,
						fill	= tk.BOTH )

		self.timeEvent()

	def	timeEvent( self ):
		th			= threading.Thread( target = self.update )
		th.start()
		self.after( PRD_POLLING, self.timeEvent )

	def	update( self ):
		get_aec_avg	( self )
		if( aec_man == AEC_AUTO ):
			get_aec_exp	( self )
		if( agc_man == AGC_AUTO ):
			get_agc_gain( self )


if __name__ == "__main__":
	root		= tk.Tk()
	app			= Application( master = root )
	app.mainloop()
	put_ini()


