#!/usr/bin/bash

FPGA_PATH=./fpga/keim-cvsoc-b_devkit_ghrd_q21
KERNEL_PATH=./kernel/linux-5.10.60
ROOTFS_PATH=rootfs/ubuntu-focal-armhf-standard
DEVICE_TREE=socfpga_cyclone5_keim-cvsoc-b

cp ${FPGA_PATH}/software/bootloader/u-boot-socfpga/u-boot-with-spl.sfp .


### create data for SD FAT partition
if [ -d ./sdfs ]; then
    rm -rf ./sdfs
fi

mkdir sdfs

cp ${FPGA_PATH}.zip ./sdfs/.
cp ${FPGA_PATH}/output_files/soc_system.rbf ./sdfs/.
cp ${FPGA_PATH}/software/bootloader/u-boot-socfpga/arch/arm/dts/socfpga_cyclone5_socdk.dtb ./sdfs/.
cp -r ${FPGA_PATH}/linux ./sdfs/.
cp ${KERNEL_PATH}/arch/arm/boot/zImage ./sdfs/.
cp ${KERNEL_PATH}/arch/arm/boot/dts/${DEVICE_TREE}_vfw.dtb ./sdfs/${DEVICE_TREE}.dtb

###### create U-Boot script file
cd ./sdfs
echo 'fatload mmc 0:1 $fdt_addr_r soc_system.rbf;' > u-boot.txt
echo 'fpga load 0 $fileaddr $filesize;' >> u-boot.txt
echo 'bridge enable;' >> u-boot.txt
echo 'mw.l 0xFF220140 0x1 1;' >> u-boot.txt
echo 'mw.l 0xFF220140 0x0 1;' >> u-boot.txt

echo '#!/usr/bin/bash' > u-boot.sh
echo 'mkimage  -A arm -O linux -T script -C none -a 0 -e 0 -n "My script" -d u-boot.txt u-boot.scr' >> u-boot.sh
sudo chmod +x u-boot.sh

./u-boot.sh

###### create linux configuration script file
if [ ! -d ./extlinux ]; then
    mkdir extlinux
fi

cd extlinux
echo 'LABEL Linux Default' > extlinux.conf
echo '    KERNEL ../zImage' >> extlinux.conf
echo '    FDT ../socfpga_cyclone5_keim-cvsoc-b.dtb' >> extlinux.conf
echo '    APPEND root=/dev/mmcblk0p2 rw rootwait earlyprintk console=ttyS0,115200n8 uio_pdrv_genirq.of_id=generic-uio' >> extlinux.conf

cd ../..

#### copy kernel module to rootfs
#sudo rm -rf ${ROOTFS_PATH}/lib/modules/*
#sudo cp -Lpr ${KERNEL_PATH}/modules_install/lib/modules/* ${ROOTFS_PATH}/lib/modules
##sudo cp -pr ${KERNEL_PATH}/modules_install/lib/modules/* ${ROOTFS_PATH}/lib/modules

### make SD image
sudo python3 ./make_sdimage_p3.py -f \
	-P u-boot-with-spl.sfp,num=3,format=raw,size=10M,type=A2  \
	-P sdfs/*,num=1,format=fat32,size=100M \
	-P ${ROOTFS_PATH}/*,num=2,format=ext3,size=7800M \
	-s 8000M \
	-n keim-vfw-focal-5.10.60.img
	
