//==============================================================================
//	File Name	: AM_MASTER.v
//	Version		: 1.3
//	Date		: 2021/06/08
//	Model		: AM_MASTER
//
//==============================================================================

module AM_MASTER #(
	parameter	integer 				BUS_WID=32'd256
)(
	input								RST_N				,
	input								AMRST_N				,
	// -- From BURST_BUFFER -- //
	input	wire						AVCLK				,
	input	wire						IWREQ				,
	input	wire						IFRM_LAST			,
//	input	wire	[3:0]				IWNUM				,
	output	wire						OBUF_RE				,
	output	wire	[3:0]				OBUF_RADDR			,
	input	wire	[255:0]				IBUF_RDT			,
	output	wire						OWACK				,
	// -- To HPS -- //
	input	wire						AMCLK				,
	output	wire						OAM_WR				,
	output	wire	[(BUS_WID-1):0]		OAM_WDATA			,
	output	wire	[31:0]				OAM_ADDR			,
	output	wire	[10:0]				OAM_BCNT			,
	output	wire	[(BUS_WID/8-1):0]	OAM_BYEN			,
	input	wire						IAM_WAITREQ			,
	// -- Register -- //
//	input	wire						IREG_WEN			,
	input	wire	[31:0]				IREG_BASE_ADDR		,
	output	wire						OREG_WDNBNK_VLD		,
	output	wire	[1:0]				OREG_WDNBNK
);

//==============================================================================
// Parameter
//==============================================================================
//parameter	BUS_WID			=	256;	//256bit


//==============================================================================
// Register & Wire
//==============================================================================
reg					fifo_emp_av_pre		;
reg					fifo_emp_av			;
reg					wr_req				;
reg					wr_req_tr			;
reg					buf_rd_que			;
wire				buf_rstr_w			;
wire				buf_rend_w			;
reg					buf_re				;
//reg		[3:0]		buf_rsize			;
reg					am_last				;
reg		[3:0]		buf_raddr			;
reg					buf_re_s1			;
reg					buf_rvld			;
reg					buf_rend			;
reg		[3:0]		buf_rend_d			;
reg					wack				;

wire	[2:0]		rd_cyc_max			;
reg		[2:0]		rd_cyc_cnt			;
reg		[2:0]		rd_cyc_cnt_s1		;
reg		[2:0]		rd_cyc_cnt_s2		;
reg		[255:0]		buf_r_dt			;
reg					fifo_we				;
wire	[255:0]		fifo_wdata			;

wire				fifo_re				;
wire	[255:0]		fifo_rdata			;
wire				fifo_emp			;
reg		[31:0]		base_addr_pre		;
reg		[31:0]		base_addr			;
reg					am_last_am_pre		;
reg					am_last_am			;
reg					buf_re_am_pre		;
reg					buf_re_am			;
reg					buf_re_am_s1		;

reg					fifo_emp_s1			;
reg					fifo_emp_s2			;
reg					fifo_emp_s3			;
reg					am_wstr_que			;
reg					am_wstr				;
reg					fifo_rbusy			;
reg					fifo_rbusy_s1		;
reg					fifo_rbusy_tf		;
reg		[3:0]		fifo_rcnt			;
reg					am_wr				;
wire	[10:0]		am_bcnt_w			;
reg		[1:0]		am_base_cnt_pre		;
wire	[31:0]		am_base_pre_w		;
reg		[31:0]		am_base_pre			;
reg		[31:0]		am_addr				;
reg					am_last_am_ld		;
reg					am_wstr_1st			;
reg		[1:0]		am_base_cnt			;
reg		[1:0]		wdnbnk_pre			;
reg		[1:0]		wdnbnk				;
reg					wdnbnk_vld_pre		;
reg		[2:0]		wdnbnk_vld_pre_d	;
wire				wdnbnk_vld_pre_d_or_w;
reg					wdnbnk_vld			;


//==============================================================================
// Description of Circuit
//==============================================================================
//----------------------------------------------------------
// FIFO Input
//----------------------------------------------------------

// FIFO_EMPTY Synchronous
always @(posedge AVCLK or negedge RST_N) begin
	if (~RST_N) begin
			fifo_emp_av_pre	<= 1'b0;
			fifo_emp_av		<= 1'b0;
	end else begin
			fifo_emp_av_pre	<= fifo_emp;
			fifo_emp_av		<= fifo_emp_av_pre;
	end
end


// WRITE REQ
always @(posedge AVCLK or negedge RST_N) begin
	if (~RST_N) begin
			wr_req		<= 1'b0;
			wr_req_tr	<= 1'b0;
	end
	else begin
			wr_req		<= IWREQ;
			wr_req_tr	<= IWREQ & ~wr_req;
	end
end



// BUFFER READ QUE
always @(posedge AVCLK or negedge RST_N) begin
	if (~RST_N) 									buf_rd_que		<= 1'b0;
	// Queue input
	else if (wr_req_tr)								buf_rd_que		<= 1'b1;
	// Remove the queue as Buffer reading starts
	else if (fifo_emp_av & ~buf_re & ~buf_rvld)		buf_rd_que		<= 1'b0;	// When fifo is empty and read is completed
	else											buf_rd_que		<= buf_rd_que;
end

// BUFFER READ START
assign buf_rstr_w = fifo_emp_av & ~buf_re & ~buf_rvld & buf_rd_que;
// BUFFER READ END
assign buf_rend_w = (buf_re & rd_cyc_cnt==rd_cyc_max & buf_raddr==4'd15)? 1'b1: 1'b0;

// BUFFER READ
always @(posedge AVCLK or negedge RST_N) begin
	if (~RST_N)										buf_re		<= 1'b0;
	else if (buf_rstr_w)							buf_re		<= 1'b1;
	else if (buf_rend_w)
													buf_re		<= 1'b0;
	else											buf_re		<= buf_re;
end


// Last of frame (DDR BANK switching)
always @(posedge AVCLK or negedge RST_N) begin
	if (~RST_N) begin
//			buf_rsize	<= 4'd0;
			am_last		<= 1'b0;
	end else if (buf_rstr_w) begin	// When Buffer read (=fifo write) starts
//			buf_rsize	<= IWNUM;
			am_last		<= IFRM_LAST;
	end else begin
//			buf_rsize	<= buf_rsize;
			am_last		<= am_last;
	end
end

// BUFFER READ ADDRESS
always @(posedge AVCLK or negedge RST_N) begin
	if (~RST_N)				buf_raddr		<= 4'd0;
	else if (buf_re & rd_cyc_cnt==rd_cyc_max)
							buf_raddr		<= buf_raddr + 4'd1;
	else					buf_raddr		<= 4'd0;
end

assign OBUF_RE    = buf_re;
assign OBUF_RADDR = buf_raddr;



// WRITE ACK
always @(posedge AVCLK or negedge RST_N) begin
	if (~RST_N) begin		buf_rend		<= 1'b0;
							buf_rend_d		<= 4'd0;
							wack			<= 1'b0;
	end
	else begin				buf_rend		<= buf_rend_w;
							buf_rend_d		<= {buf_rend_d[2:0], buf_rend};
							wack			<= buf_rend_d[3];
	end
end

assign	OWACK = wack;

// BUFFER READ VALID
always @(posedge AVCLK or negedge RST_N) begin
	if (~RST_N) begin
			buf_re_s1		<= 1'b0;
			buf_rvld		<= 1'b0;
	end
	else begin
			buf_re_s1		<= buf_re;
			buf_rvld		<= buf_re_s1;
	end
end



//----------------------------------------------------------
// Bus width conversion
//----------------------------------------------------------
// Read cycle
assign rd_cyc_max   = (BUS_WID==32'd256)? 3'd0 :
					  (BUS_WID==32'd128)? 3'd1 :
					  (BUS_WID==32'd64)?  3'd3 :
					  (BUS_WID==32'd32)?   3'd7 : 3'd0;
					  

always @(posedge AVCLK or negedge RST_N) begin
	if (~RST_N)										rd_cyc_cnt	<= 3'd0;
	else if (buf_rstr_w)							rd_cyc_cnt	<= 3'd0;
	else if (buf_re & rd_cyc_cnt==rd_cyc_max)		rd_cyc_cnt	<= 3'd0;
	else											rd_cyc_cnt	<= rd_cyc_cnt + 3'd1;
end

always @(posedge AVCLK or negedge RST_N) begin
	if (~RST_N) begin
			rd_cyc_cnt_s1	<= 3'd0;
			rd_cyc_cnt_s2	<= 3'd0;
	end
	else begin
			rd_cyc_cnt_s1	<= rd_cyc_cnt;
			rd_cyc_cnt_s2	<= rd_cyc_cnt_s1;
	end
end

always @(posedge AVCLK or negedge RST_N) begin
	if (~RST_N)						buf_r_dt	<= 256'd0;
	else if (buf_rvld)
		if (rd_cyc_cnt_s2==3'd0)	buf_r_dt	<= IBUF_RDT;
		else
			if (BUS_WID==32'd256)			buf_r_dt	<= 256'd0;
			else if (BUS_WID==32'd128)		buf_r_dt	<= {128'd0, buf_r_dt[255:128]};
			else if (BUS_WID==32'd64)		buf_r_dt	<= {64'd0,  buf_r_dt[255:64]};
			else							buf_r_dt	<= {32'd0,  buf_r_dt[255:32]};	// BIS_WID==32'd32
	else							buf_r_dt	<= 256'd0;
end

assign fifo_wdata = buf_r_dt;

always @(posedge AVCLK or negedge RST_N) begin
	if (~RST_N)			fifo_we		<= 1'b0;
	else				fifo_we		<= buf_rvld;
end


//----------------------------------------------------------
// FIFO
//----------------------------------------------------------

// Write : AVCLK

// Read : AMCLK

dcfifo fifo16x256 (
	.aclr		(~RST_N		),	// input		aclr;
	.data		(fifo_wdata	),	// input	[255:0]  data;
	.rdclk		(AMCLK		),	// input	  rdclk;
	.rdreq		(fifo_re	),	// input	  rdreq;
	.wrclk		(AVCLK		),	// input	  wrclk;
	.wrreq		(fifo_we	),	// input	  wrreq;
	.q			(fifo_rdata	),	// output	[255:0]  q;
	.rdempty	(fifo_emp	),	// output	  rdempty;
	.wrfull		(			) 	// output	  wrfull;
	);

	defparam
		fifo16x256.intended_device_family = "Cyclone V",
		fifo16x256.lpm_numwords = 16,
		fifo16x256.lpm_showahead = "OFF",
		fifo16x256.lpm_type = "dcfifo",
		fifo16x256.lpm_width = 256,
		fifo16x256.lpm_widthu = 4,
		fifo16x256.overflow_checking = "ON",
		fifo16x256.rdsync_delaypipe = 4,
		fifo16x256.read_aclr_synch = "OFF",
		fifo16x256.underflow_checking = "ON",
		fifo16x256.use_eab = "ON",
		fifo16x256.write_aclr_synch = "OFF",
		fifo16x256.wrsync_delaypipe = 4;

assign fifo_re = am_wstr | (~IAM_WAITREQ & fifo_rbusy & ~fifo_emp); // Only the first read cycle is performed regardless of waitreq

//----------------------------------------------------------
// Avalon_MM Write
//----------------------------------------------------------
// Input FF (CDC)
always @(posedge AMCLK or negedge AMRST_N) begin
	if (~AMRST_N) begin		base_addr_pre	<= 32'd0;
							base_addr		<= 32'd0;
							am_last_am_pre	<= 1'b0;
							am_last_am		<= 1'b0;
							buf_re_am_pre	<= 1'b0;
							buf_re_am		<= 1'b0;
							buf_re_am_s1	<= 1'b0;
	end else begin			base_addr_pre	<= IREG_BASE_ADDR;
							base_addr		<= base_addr_pre;
							am_last_am_pre	<= am_last;
							am_last_am		<= am_last_am_pre;
							buf_re_am_pre	<= buf_re;
							buf_re_am		<= buf_re_am_pre;
							buf_re_am_s1	<= buf_re_am;
	end
end


always @(posedge AMCLK or negedge AMRST_N) begin
	if (~AMRST_N) begin
			fifo_emp_s1	<= 1'b0;
			fifo_emp_s2	<= 1'b0;
			fifo_emp_s3	<= 1'b0;
	end else begin
			fifo_emp_s1	<= fifo_emp;
			fifo_emp_s2	<= fifo_emp_s1;
			fifo_emp_s3	<= fifo_emp_s2;
	end
end

// Write Start
// Ensure that "am_wstr" is not set more than once
// in case FIFO read is too fast and Empty occurs during FIFO write.

always @(posedge AMCLK or negedge AMRST_N) begin
	if (~AMRST_N)							am_wstr_que	<= 1'b0;
	else if (buf_re_am & ~buf_re_am_s1)		am_wstr_que	<= 1'b1;	// Start buffer read
	else if (~fifo_emp_s2 & fifo_emp_s3)	am_wstr_que	<= 1'b0;	// Start write
	else									am_wstr_que	<= am_wstr_que;
end

always @(posedge AMCLK or negedge AMRST_N) begin
	if (~AMRST_N)							am_wstr		<= 1'b0;
	else if (~fifo_emp_s2 & fifo_emp_s3)	am_wstr		<= am_wstr_que;	// Start write
	else									am_wstr		<= 1'b0;
end

// Write BUSY
always @(posedge AMCLK or negedge AMRST_N) begin
	if (~AMRST_N)										fifo_rbusy	<= 1'b0;
	else if (~fifo_emp_s2 & fifo_emp_s3 & am_wstr_que)	fifo_rbusy	<= 1'b1;
	else if (fifo_re & fifo_rcnt==4'd15)				fifo_rbusy	<= 1'b0;
	else												fifo_rbusy	<= fifo_rbusy;
end

always @(posedge AMCLK or negedge AMRST_N) begin
	if (~AMRST_N)										fifo_rcnt	<= 4'd0;
	else if (fifo_re)
		if (fifo_rcnt==4'd15)							fifo_rcnt	<= 4'd0;
		else											fifo_rcnt	<= fifo_rcnt + 4'd1;
	else												fifo_rcnt	<= fifo_rcnt;
end

always @(posedge AMCLK or negedge AMRST_N) begin
	if (~AMRST_N)										fifo_rbusy_s1	<= 1'b0;
	else if (~IAM_WAITREQ)								fifo_rbusy_s1	<= fifo_rbusy;
	else												fifo_rbusy_s1	<= fifo_rbusy_s1;
end


always @(posedge AMCLK or negedge AMRST_N) begin
	if (~AMRST_N)										fifo_rbusy_tf	<= 1'b0;
	else if (~IAM_WAITREQ)								fifo_rbusy_tf	<= ~fifo_rbusy & fifo_rbusy_s1;
	else												fifo_rbusy_tf	<= fifo_rbusy_tf;
end


// Write 
always @(posedge AMCLK or negedge AMRST_N) begin
	if (~AMRST_N)											am_wr	<= 1'b0;
	else if (fifo_rbusy & ~fifo_emp)						am_wr	<= 1'b1;
	else if (~fifo_rbusy & fifo_rbusy_s1 & IAM_WAITREQ)		am_wr	<= 1'b1;//If waitreq is set at the end, extend
	else													am_wr	<= 1'b0;
end


//Fixed to 16 this time
assign am_bcnt_w[10:0] = 11'd16;


// Write Address
always @(posedge AMCLK or negedge AMRST_N) begin
	if (~AMRST_N)			am_last_am_ld	<= 1'b0;
	else if (am_wstr)		am_last_am_ld	<= am_last_am;
	else					am_last_am_ld	<= am_last_am_ld;
end

always @(posedge AMCLK or negedge AMRST_N) begin
	if (~AMRST_N)			am_wstr_1st		<= 1'b1;
	else if (am_wstr)		am_wstr_1st		<= 1'b0;
	else					am_wstr_1st		<= am_wstr_1st;
end


// Bank to be written in the next frame
always @(posedge AMCLK or negedge AMRST_N) begin
	if (~AMRST_N)						am_base_cnt_pre	<= 2'b00;
//	else if (am_wstr & am_last_am_ld)
	else if (am_wstr & (am_wstr_1st | am_last_am_ld))
		if (am_base_cnt_pre == 2'b10)	am_base_cnt_pre	<= 2'b00;
		else							am_base_cnt_pre	<= am_base_cnt_pre + 2'd1;
	else								am_base_cnt_pre	<= am_base_cnt_pre;
end

// Base address of next frame
assign am_base_pre_w 	=	(am_base_cnt_pre==2'b00)? base_addr :
							(am_base_cnt_pre==2'b01)? base_addr + 32'hF0000 :	//8bit/word
							base_addr + 32'h1E0000 ;

always @(posedge AMCLK or negedge AMRST_N) begin
	if (~AMRST_N)		am_base_pre		<= 32'd0;
	else				am_base_pre		<= am_base_pre_w;
end

// Current Address (Next->Current)
always @(posedge AMCLK or negedge AMRST_N) begin
	if (~AMRST_N)			am_addr			<= 32'd0;
	else if (am_wstr)
		if (am_wstr_1st | am_last_am_ld)	// Update the base for the first time after RST
							am_addr			<= am_base_pre;		
		else				am_addr			<= am_addr + 32'd512;	//16burst x 256bit=512 x 8bit
	else					am_addr			<= am_addr;
end


assign OAM_WR    = am_wr;
assign OAM_WDATA = fifo_rdata[(BUS_WID-1):0];
assign OAM_BCNT  = am_bcnt_w;
assign OAM_ADDR  = am_addr;
assign OAM_BYEN  = {(BUS_WID/8){1'b1}};


//----------------------------------------------------------
// Write completion bank output
//----------------------------------------------------------
// Curren Bank (Next->Current)
always @(posedge AMCLK or negedge AMRST_N) begin
	if (~AMRST_N)						am_base_cnt	<= 2'b00;
	else if (am_wstr & am_last_am_ld)	am_base_cnt	<= am_base_cnt_pre;
	else								am_base_cnt	<= am_base_cnt;
end

// Previous bank (Current->Previous)
always @(posedge AMCLK or negedge AMRST_N) begin
	if (~AMRST_N) begin
				wdnbnk_pre		<= 2'b00;
				wdnbnk_vld_pre	<= 1'b0;
	end
	else if (fifo_rbusy_tf & am_last_am_ld) begin	//Change to last write
				wdnbnk_pre		<= am_base_cnt;
				wdnbnk_vld_pre	<= 1'b1;
	end
	else begin
				wdnbnk_pre		<= wdnbnk_pre;
				wdnbnk_vld_pre	<= 1'b0;
	end
end

always @(posedge AMCLK or negedge AMRST_N) begin
	if (~AMRST_N)		wdnbnk_vld_pre_d	<= 3'b0;
	else			wdnbnk_vld_pre_d	<= {wdnbnk_vld_pre_d[1:0], wdnbnk_vld_pre};
end

assign wdnbnk_vld_pre_d_or_w = {wdnbnk_vld_pre | wdnbnk_vld_pre_d[0] |  wdnbnk_vld_pre_d[1] |  wdnbnk_vld_pre_d[2]};

always @(posedge AMCLK or negedge AMRST_N) begin
	if (~AMRST_N) begin
				wdnbnk		<= 2'b0;
				wdnbnk_vld	<= 1'b0;
	end
	else begin
				wdnbnk		<= wdnbnk_pre;
				wdnbnk_vld	<= wdnbnk_vld_pre_d_or_w;
	end
end

assign OREG_WDNBNK     = wdnbnk;
assign OREG_WDNBNK_VLD = wdnbnk_vld;

endmodule
