//==============================================================================
//	File Name	: AV_SINK.v
//	Version		: 1.0
//	Date		: 2010/12/20
//	Model		: AV_SINK
//
//==============================================================================

module AV_SINK (
	input	wire				RST_N		,
	input	wire				AVCLK		,
	// -- Avalon-ST Video input -- //
	input	wire				IAV_STR		,
	input	wire				IAV_END		,
	input	wire	[23:0]		IAV_DT		,
	input	wire				IAV_VLD		,
//	input	wire	[1:0]		IAV_EMP		,
	output						OAV_RDY		,
	// -- To Buffer -- //
	output	wire				OV_LAST		,
	output	wire	[23:0]		OV_DT		,
	output	wire				OV_VLD		,
	input	wire				IV_RDY		,
	// -- Register -- //
	input	wire				IREG_WEN
);

//==============================================================================
// Parameter
//==============================================================================

//==============================================================================
// Register & Wire
//==============================================================================
reg					av_str		;
reg					av_end		;
reg		[23:0]		av_dt		;
reg					av_vld		;
//reg		[1:0]		av_emp		;
reg					wen_pre		;
reg					wen			;

reg					vd_pkt		;
reg					v_vld		;
reg		[23:0]		v_dt		;
reg					v_last		;

reg					av_rdy		;
reg					av_rdy_s1	;

//==============================================================================
// Description of Circuit
//==============================================================================

// Input FF
always @(posedge AVCLK or negedge RST_N) begin
	if (~RST_N) begin
		av_str		<= 1'b0;
		av_end		<= 1'b0;
		av_dt		<= 24'd0;
		av_vld		<= 1'b0;
//		av_emp		<= 1'b0;
	end
	else begin
		av_str		<= IAV_STR;
		av_end		<= IAV_END;
		av_dt		<= IAV_DT;
		av_vld		<= IAV_VLD & av_rdy_s1;	// latency 1
//		av_emp		<= IAV_EMP;
	end
end

// Input FF(CDC)
always @(posedge AVCLK or negedge RST_N) begin
	if (~RST_N) begin
		wen_pre		<= 1'b0;
		wen			<= 1'b0;
	end
	else begin
		wen_pre		<= IREG_WEN;
		wen			<= wen_pre;
	end
end


// --------------------------------------------------------- 
// Classification of Video data packet
// --------------------------------------------------------- 
always @(posedge AVCLK or negedge RST_N) begin
	if (~RST_N)							vd_pkt	<= 1'b0;
	else
		if (av_str & av_vld & wen)
			if (av_dt==24'd0) 				vd_pkt	<= 1'b1;
			else							vd_pkt	<= 1'b0;
		else if (av_end & av_vld)
										vd_pkt	<= 1'b0;
		else							vd_pkt	<= vd_pkt;
end


// --------------------------------------------------------- 
// Extraction of Video data packet
// --------------------------------------------------------- 
always @(posedge AVCLK or negedge RST_N) begin
	if (~RST_N) begin
				v_vld	<= 1'b0;
				v_dt	<= 24'd0;
				v_last	<= 1'b0;
	end
	else begin
		if (vd_pkt) begin
				v_vld	<= av_vld;
				v_dt	<= av_dt;
				v_last	<= av_end;
		end
		else begin
				v_vld	<= 1'b0;
				v_dt	<= 24'd0;
				v_last	<= 1'b0;
		end
	end
end

assign OV_LAST = v_last;
assign OV_DT   = v_dt;
assign OV_VLD  = v_vld;


// --------------------------------------------------------- 
// OAV_RDY
// --------------------------------------------------------- 
always @(posedge AVCLK or negedge RST_N) begin
	if (~RST_N) begin
			av_rdy		<= 1'b0;
			av_rdy_s1	<= 1'b0;
	end else begin
			av_rdy		<= IV_RDY;
			av_rdy_s1	<= av_rdy;
	end
end

assign OAV_RDY = av_rdy;


endmodule
