//==============================================================================
//	File Name	: VFW_TOP.v
//	Version		: 1.0
//	Date		: 2021/05/18
//	Model		: VFW_TOP
//
//==============================================================================

module VFW_TOP #(
	parameter	integer						AM_BUS_WID = 32'd256
)(
	// -- Avalon-ST Video input -- //
	input	wire							AVRST_N			,
	input	wire							AVCLK			,
	input	wire							IAV_STR			,
	input	wire							IAV_END			,
	input	wire	[23:0]					IAV_DT			,
	input	wire							IAV_VLD			,
//	input	wire	[1:0]					IAV_EMP			,
	output									OAV_RDY			,
	// -- To AM_MASER -- //
	input	wire							AMRST_N			,
	input	wire							AMCLK			,
	output	wire							OAM_WR			,
	output	wire	[(AM_BUS_WID-1):0]		OAM_WDATA		,
	output	wire	[31:0]					OAM_ADDR		,
	output	wire	[10:0]					OAM_BCNT		,
	output	wire	[(AM_BUS_WID/8-1):0]	OAM_BYEN		,
	input									IAM_WAITREQ		,
	// -- Avalon_MM Slave -- //
	input	wire							ASRST_N			,
	input	wire							ASCLK			,
	input	wire							IAS_WR			,
	input	wire	[31:0]					IAS_WDATA		,
	input	wire							IAS_RD			,
	output	wire	[31:0]					OAS_RDATA		,
	output	wire							OAS_RVLD		,
	input	wire	[1:0]					IAS_ADDR		,
	input	wire	[3:0]					IAS_BYEN		,
	output	wire							OAS_WAITREQ
);

//==============================================================================
// Parameter
//==============================================================================
//parameter AM_BUS_WID = 256;

//==============================================================================
// Register & Wire
//==============================================================================
wire				v_last			;
wire	[23:0]		v_dt			;
wire				v_vld			;
wire				v_rdy			;

wire				wreq			;
wire				frm_last		;
//wire	[3:0]		wnum			;
wire				wack			;
wire				buf_re			;
wire	[3:0]		buf_raddr		;
wire	[255:0]		buf_rdt			;

wire				reg_wen			;
wire				reg_wdnbnk_vld	;
wire	[1:0]		reg_wdnbnk		;
wire	[31:0]		reg_base_addr	;


//==============================================================================
// Description of Circuit
//==============================================================================

AV_SINK av_sink (
	.RST_N			(AVRST_N		),		// input	wire
	.AVCLK			(AVCLK			),		// input	wire
	// -- Avalon-ST Video input -- //
	.IAV_STR		(IAV_STR		),		// input	wire
	.IAV_END		(IAV_END		),		// input	wire
	.IAV_DT			(IAV_DT			),		// input	wire	[23:0]
	.IAV_VLD		(IAV_VLD		),		// input	wire
//	.IAV_EMP		(IAV_EMP		),		// input	wire	[1:0]
	.OAV_RDY		(OAV_RDY		),		// output
	// -- To Buffer -- //
	.OV_LAST		(v_last			),		// output	wire
	.OV_DT			(v_dt			),		// output	wire	[23:0]
	.OV_VLD			(v_vld			),		// output	wire
	.IV_RDY			(v_rdy			),		// input	wire
	// -- Register -- //
	.IREG_WEN	 	(reg_wen		) 		// input	wire
);

BURST_BUFFER bbuffer (
	.RST_N			(AVRST_N		),		// input	wire
	.AVCLK			(AVCLK			),		// input	wire
	// -- From AV_SINK -- //
	.IV_VLD			(v_vld			),		// input	wire
	.IV_DT			(v_dt			),		// input	wire	[23:0]
	.IV_LAST		(v_last			),		// input	wire
	.OV_RDY			(v_rdy			),		// output	wire
	// -- To AM_MASER -- //
	.OWREQ			(wreq			),		// output	wire
	.OFRM_LAST		(frm_last		),		// output	wire
//	.OWNUM			(wnum			),		// output	wire	[3:0]
	.IWACK			(wack			),		// input	wire
	.IBUF_RE		(buf_re			),		// input	wire
	.IBUF_RADDR		(buf_raddr		),		// input	wire	[3:0]
	.OBUF_RDT	 	(buf_rdt		) 		// output	wire	[255:0]
);

AM_MASTER #(
	.BUS_WID			(AM_BUS_WID		))		// Avalon_MM-Master Bus Width
am_master (
	.RST_N				(AVRST_N		),		// input
	.AMRST_N			(AMRST_N		),		// input
	// -- From BURST_BUFFER -- //
	.AVCLK				(AVCLK			),		// input	wire
	.IWREQ				(wreq			),		// input	wire
	.IFRM_LAST			(frm_last		),		// input	wire
//	.IWNUM				(wnum			),		// input	wire	[3:0]
	.OBUF_RE			(buf_re			),		// output	wire
	.OBUF_RADDR			(buf_raddr		),		// output	wire	[3:0]
	.IBUF_RDT			(buf_rdt		),		// input	wire	[255:0]
	.OWACK				(wack			),		// output	wire
	// -- To HPS -- //
	.AMCLK				(AMCLK			),		// input	wire
	.OAM_WR				(OAM_WR			),		// output	wire
	.OAM_WDATA			(OAM_WDATA		),		// output	wire	[(BUS_WID-1):0]
	.OAM_ADDR			(OAM_ADDR		),		// output	wire	[31:0]
	.OAM_BCNT			(OAM_BCNT		),		// output	wire	[10:0]
	.OAM_BYEN			(OAM_BYEN		),		// output	wire	[(BUS_WID/8-1):0]
	.IAM_WAITREQ		(IAM_WAITREQ	),		// input	wire
	// -- Register -- //
	.IREG_BASE_ADDR		(reg_base_addr	),		// input	wire	[31:0]
	.OREG_WDNBNK_VLD	(reg_wdnbnk_vld	),		// output	wire
	.OREG_WDNBNK		(reg_wdnbnk		) 		// output	wire	[1:0]
);

AM_SLAVE am_slave(
	.RST_N				(ASRST_N		),		// input	wire
	// -- Avalon_MM Slave -- //
	.ASCLK				(ASCLK			),		// input	wire
	.IAS_WR				(IAS_WR			),		// input	wire
	.IAS_WDATA			(IAS_WDATA		),		// input	wire	[31:0]
	.IAS_RD				(IAS_RD			),		// input	wire
	.OAS_RDATA			(OAS_RDATA		),		// output	wire	[31:0]
	.OAS_RVLD			(OAS_RVLD		),		// output	wire
	.IAS_ADDR			(IAS_ADDR		),		// input	wire	[1:0]
	.IAS_BYEN			(IAS_BYEN		),		// input	wire	[3:0]
	.OAS_WAITREQ		(OAS_WAITREQ	),		// output	wire
	// -- Register -- //
	.OREG_WEN			(reg_wen		),		// output
	.IREG_WDNBNK_VLD	(reg_wdnbnk_vld	),		// input
	.IREG_WDNBNK		(reg_wdnbnk		),		// input	[1:0]
	.OREG_BASE_ADDR	 	(reg_base_addr	) 		// output	[31:0]
);



endmodule
