//==============================================================================
//	File Name	: VFW_TOP_wrapper.v
//	Version		: 1.0
//	Date		: 2021/05/18
//	Model		: VFW_TOP_wrapper
//
//==============================================================================

module VFW_TOP_wrapper #(
	parameter	integer						AM_BUS_WID = 256
)(
	// -- Avalon-ST Video input -- //
	input	wire							asi_reset_reset		,
	input	wire							asi_clock_clk		,
	input	wire							asi_startofpacket	,
	input	wire							asi_endofpacket		,
	input	wire	[23:0]					asi_data			,
	input	wire							asi_valid			,
//	input	wire	[1:0]					asi_empty			,
	output	wire							asi_ready			,
	// -- To AM_MASER -- //
	input	wire							avm_reset_reset		,
	input	wire							avm_clock_clk		,
	output	wire							avm_write			,
	output	wire	[(AM_BUS_WID-1):0]		avm_writedata		,
	output	wire	[31:0]					avm_address			,
	output	wire	[10:0]					avm_burstcount		,
	output	wire	[(AM_BUS_WID/8-1):0]	avm_byteenable		,
	input	wire							avm_waitrequest		,
	// -- Avalon_MM Slave -- //
	input	wire							avs_reset_reset		,
	input	wire							avs_clock_clk		,
	input	wire							avs_write			,
	input	wire	[31:0]					avs_writedata		,
	input	wire							avs_read			,
	output	wire	[31:0]					avs_readdata		,
	output	wire							avs_readdatavalid	,
	input	wire	[1:0]					avs_address			,
	input	wire	[3:0]					avs_byteenable		,
	output	wire							avs_waitrequest
);

//==============================================================================
// Parameter
//==============================================================================
//parameter AM_BUS_WID = 256;

//==============================================================================
// Description of Circuit
//==============================================================================

VFW_TOP #(.AM_BUS_WID (AM_BUS_WID)) 
vfw_top (
	// -- Avalon-ST Video input -- //	74.25MHz
	.AVRST_N		(~asi_reset_reset	),	// input	wire
	.AVCLK			(asi_clock_clk		),	// input	wire
	.IAV_STR		(asi_startofpacket	),	// input	wire
	.IAV_END		(asi_endofpacket	),	// input	wire
	.IAV_DT			(asi_data			),	// input	wire	[23:0]
	.IAV_VLD		(asi_valid			),	// input	wire
//	.IAV_EMP		(asi_empty			),	// input	wire	[1:0]
	.OAV_RDY		(asi_ready			),	// output
	// -- To AM_MASER -- //				74.25MHz
	.AMRST_N		(~avm_reset_reset	),	// input	wire
	.AMCLK			(avm_clock_clk		),	// input	wire
	.OAM_WR			(avm_write			),	// output	wire
	.OAM_WDATA		(avm_writedata		),	// output	wire	[(AM_BUS_WID-1):0]
	.OAM_ADDR		(avm_address		),	// output	wire	[31:0]
	.OAM_BCNT		(avm_burstcount		),	// output	wire	[10:0]
	.OAM_BYEN		(avm_byteenable		),	// output	wire	[(AM_BUS_WID/8-1):0]
	.IAM_WAITREQ	(avm_waitrequest	),	// input	wire
	// -- Avalon_MM Slave -- //			50MHz
	.ASRST_N		(~avs_reset_reset	),	// input	wire
	.ASCLK			(avs_clock_clk		),	// input	wire
	.IAS_WR			(avs_write			),	// input	wire
	.IAS_WDATA		(avs_writedata		),	// input	wire	[31:0]					
	.IAS_RD			(avs_read			),	// input	wire
	.OAS_RDATA		(avs_readdata		),	// output	wire	[31:0]					
	.OAS_RVLD		(avs_readdatavalid	),	// output	wire
	.IAS_ADDR		(avs_address		),	// input	wire	[1:0]
	.IAS_BYEN		(avs_byteenable		),	// input	wire	[3:0]
	.OAS_WAITREQ	(avs_waitrequest	)	// output	wire
);



endmodule
